# -*- coding: utf-8 -*-
#
# @copyright &copy; 2012 by Roberto Rocco Angeloni <roberto@roccoangeloni.it>
# @license http://opensource.org/licenses/lgpl-3.0.html GNU Lesser General Public License, version 3.0 (LGPLv3)
# @version $Id: xmlrpc.py $
# @package dblayer
# 
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

from rprj.dblayer import DBConnectionProvider,DBLayerException,isDateTime
import codecs,datetime,os,sys,traceback
from base64 import b64decode

import pymongo

# 1 mega = 1048576
UPLOAD_SPLIT_SIZE = 1048576


class MongoConnectionProvider(DBConnectionProvider):
	def __init__(self, host, db, user, pwd, verbose=0):
		"""host: here is the url of the server
		db: TODO is meaningless here?"""
		DBConnectionProvider.__init__(self, host, db, user, pwd, verbose)
		self.verbose = verbose
		self._server = None
		self._conn = None
		self._cache = {}
		self.dbmgr = None # Needed by _jsonToDbe
		self.user = None # Web should act as a proxy
	def customInit(self):
		"""???"""
		pass
	def getConnection(self):
		if self._conn is None:
			self._conn = pymongo.Connection(self._host)
			self._server = self._conn[self._db]
		return self
	def freeConnection(self, conn):
		if not self._conn is None:
			del self._conn
		self._server = None
	def getDBType(self):
		return "Mongo"
	def isProxy(self):
		# NOTE non è un proxy MA le query deve costruirle lui
		return True

	def _dbeToJson(self,dbe):
		ret = dbe.getValuesDictionary()
		if ret.has_key('id'):
			ret['_id']=ret['id']
			ret.pop('id')
		return ret
	def _jsonToDbe(self, tablename, dbedict):
		ret = self.dbmgr.getClazz(tablename)()
		chiavi = ret.getKeys().keys()
		for k in dbedict.keys():
			if k=='_id' and 'id' in chiavi:
				ret.setValue('id', dbedict[k])
				continue
			ret.setValue(k, dbedict[k])
		return ret
		#typename = xmldbe['_typename']
		#xmldbe.pop('_typename')
		#nomi = [ n for n in xmldbe.keys() ]
		#valori = [ n for n in xmldbe.values() ]
		#valori2=[]
		#for v in valori:
			##if isinstance(v, Binary): v = v.data
			#if isDateTime(v): #rra.dblayer.isDateTime(v):
				##"0000-00-00 00:00:00"
				#v2=None
				#try:
					#v2 = datetime.datetime(int(v[:4]),int(v[5:7]),int(v[8:10]),int(v[11:13]),int(v[14:16]),int(v[17:19]))
				#except Exception,e:
					#try:
						#v2 = datetime.datetime(int(v[:4]),int(v[5:7]),int(v[8:10]),int(v[11:13]),int(v[14:16]))
					#except Exception, e:
						#try:
							#v2 = datetime.time(int(v[11:13]),int(v[14:16]),int(v[17:19]))
						#except Exception,e:
							#v2 = datetime.time(int(v[11:13]),int(v[14:16]))
				#valori2.append( v2 )
			#elif (type(v)==str or type(v)==unicode) and v.startswith("base64:"):
				#valori2.append( b64decode( v[len("base64:"):] ) )
			#else:
				#valori2.append(v)
		##if isinstance(typename, Binary): typename = typename.data
		#myclazz = self.dbmgr.getClazzByTypeName(typename)
		#return myclazz( myclazz().getTableName(), names=nomi, values=valori2 )
	def insert(self, dbe):
		# Before Insert
		dbe._before_insert(self.dbmgr)
		dbe = self.dbmgr._before_insert(dbe)
		# Insert
		dbejson = self._dbeToJson(dbe)
		print "dbejson:",dbejson
		collectionName = self.dbmgr.buildTableName(dbe)
		print "collectionName:",collectionName
		coll = self._server[ collectionName ]
		try:
			dbejson = coll.insert( dbejson )
		except Exception, e:
			print "ECCEZIONE: %s" % ( e )
			print "".join( traceback.format_tb(sys.exc_info()[2]) )
		print "=> dbejson:",dbejson
		#raise DBLayerException("DA FINIRE")
		self.lastMessages = self._server.error()
		#dbe = self._jsonToDbe( tmparray[0] )
		# After Insert
		dbe = self.dbmgr._after_insert(dbe)
		dbe._after_insert(self.dbmgr)
		return dbe
	def update(self,dbe):
		# Before Update
		# 2012.04.04 dbe._before_update(self.dbmgr)
		# 2012.04.04 dbe = self.dbmgr._before_update(dbe)
		# Update
		dbexml = self._dbeToJson(dbe)
		#self.semaforo.acquire(True)
		tmp = self._server.update( dbexml )
		#self.semaforo.release()
		tmparray = tmp[1]
		self.lastMessages = b64decode(tmp[0])
		if len(tmparray)>0:
			dbe = self._jsonToDbe( tmparray[0] )
			# After Update
			# 2012.04.04 dbe = self.dbmgr._after_update(dbe)
			# 2012.04.04 dbe._after_update(self.dbmgr)
		else:
			dbe = None
		return dbe
	def select(self,tablename,searchString):
		if self.verbose: print "MongoConnectionProvider.select: tablename=%s" % (tablename)
		if self.verbose: print "MongoConnectionProvider.select: searchString=%s" % (searchString)
		if self.verbose: print "MongoConnectionProvider.select: start search=%s" % (datetime.datetime.now())
		try:
			#self.semaforo.acquire(True)
			tmp = self._server.select( tablename, searchString )
			#self.semaforo.release()
			res = tmp[1]
			self.lastMessages = b64decode(tmp[0])
		except DBLayerException, e:
			raise DBLayerException( "Problemi di comunicazione col server: %s" % e )
		if self.verbose: print "MongoConnectionProvider.select: start xml->dbe=%s" % (datetime.datetime.now())
		ret = []
		for r in res:
			tmpdbe = self._jsonToDbe(r)
			ret.append( tmpdbe )
		if self.verbose: print "MongoConnectionProvider.select: end time = %s" % (datetime.datetime.now())
		return ret
	def delete(self,dbe):
		# Delete
		dbexml = self._dbeToJson(dbe)
		#self.semaforo.acquire(True)
		tmp = self._server.delete( dbexml )
		#self.semaforo.release()
		tmparray = tmp[1]
		self.lastMessages = b64decode(tmp[0])
		if self.verbose: print "MongoConnectionProvider.delete: self.lastMessages = %s" % (self.lastMessages)
		dbe = self._jsonToDbe( tmparray[0] )
		return dbe
	def search(self, dbe, uselike=1, orderby=None, ignore_deleted=True, full_object=True):
		casesensitive = 0
		if self.verbose: print "MongoConnectionProvider.search: start time = %s" % (datetime.datetime.now())
		tablename = dbe.getTableName()
		# Search
		dbedict = dbe.getValuesDictionary()
		if self.verbose: print "MongoConnectionProvider.search: dbedict = %s" % (dbedict)
		if self.verbose: print "MongoConnectionProvider.search: uselike = %s" % (uselike)
		#if self.verbose: print "MongoConnectionProvider.search: casesensitive = %s" % (casesensitive)
		if self.verbose: print "MongoConnectionProvider.search: orderby = %s" % (orderby)
		lista = []
		try:
			if orderby is None: orderby=''
			#self.semaforo.acquire(True)
			collectionName = self.dbmgr.buildTableName(dbe)
			#if not collectionName in self._server.collection_names():
			#	raise DBLayerException("Collection '%s' does not exists." % (collectionName))
			coll = self._server[ collectionName ]
			if self.verbose: print "MongoConnectionProvider.search: coll=%s (%s)\n  %s" % (coll,  coll.count(),  dir(coll))
			lista = coll.find( dbedict )
			#self.lastMessages = msg
		except Exception, e:
			print "MongoConnectionProvider.search: ECCEZIONE ############# Inizio."
			print "ECCEZIONE: %s" % ( e )
			print "".join( traceback.format_tb(sys.exc_info()[2]) )
			print "MongoConnectionProvider.search: ECCEZIONE ############# Fine."
			raise DBLayerException( "Problemi di comunicazione col server: %s" % e )
		ret = []
		if self.verbose: print "MongoConnectionProvider.search: lista=%s (%s)" % (lista, lista.count())
		for r in lista:
			if self.verbose: print "  %s" % r
			tmpdbe = self._jsonToDbe(tablename, r)
			if self.verbose: print "  %s" % tmpdbe
			ret.append( tmpdbe )
		if self.verbose: print "MongoConnectionProvider.search: ret = %s" % ( len(ret) )
		if self.verbose: print "MongoConnectionProvider.search: end time = %s" % (datetime.datetime.now())
		return ret
	def copy(self, dbe):
		raise DBLayerException("copy: not yet implemented")

# ################### Remote Server ###################
	def ping(self):
		#self.semaforo.acquire(True)
		serverinfo = self._conn.server_info()
		#self.semaforo.release()
		if self.verbose:
			print "MongoConnectionProvider.ping: serverinfo"
			for k in serverinfo.keys():
				print "%s: %s" % (k,serverinfo[k])
		return "pong"
	def initDB(self):
		"""Initialize DB"""
		if self.verbose: print "MongoConnectionProvider.initDB: start."
		mytypes = self.dbmgr.dbeFactory.getRegisteredTypes()
		for tablename in mytypes.keys():
			if tablename=='default':
				continue
			mydbe = mytypes[tablename]()
			collectionName = self.dbmgr.buildTableName(mydbe)
			if self.verbose: print "MongoConnectionProvider.initDB: collectionName = %s" % ( collectionName )
			# 1. Table definition
			try:
				self._server.create_collection(collectionName)
				dbErrors = self._server.error()
				if not dbErrors is None:
					if self.verbose: print "MongoConnectionProvider.initDB: dbErrors = %s" % dbErrors
					raise DBLayerException(dbErrors)
			except Exception,e:
				if self.verbose:
					print "MongoConnectionProvider.initDB: Exception = %s" % (e)
					print "".join(traceback.format_tb(sys.exc_info()[2]))
		# 2. Default entries
		# 2012.06.04: start.
		for tablename in mytypes.keys():
			if tablename=='default':
				continue
			mydbe = mytypes[tablename]()
			# 2012.06.04: end.
			defaultEntries = mydbe.getDefaultEntries()
			if len(defaultEntries)==0: continue
			for d in defaultEntries:
				if self.verbose: print "MongoConnectionProvider.initDB: d = %s" % (d)
				newdbe = mytypes[tablename]( attrs=d )
				self.insert(newdbe)
		if self.verbose: print "MongoConnectionProvider.initDB: end."
	def login(self,user,pwd):
		if self.verbose: print "MongoConnectionProvider.login: start."
		if pwd is None or len(pwd)==0 or user is None or len(user)==0:
			raise DBLayerException("Missing username or password")
		cerca = self.dbmgr.getClazzByTypeName('DBEUser')(attrs={'login':user,'pwd':pwd})
		ret = []
		try:
			collectionName = self.dbmgr.buildTableName(cerca)
			if not collectionName in self._server.collection_names():
				raise DBLayerException("Collection '%s' does not exists." % (collectionName))
			ret = self.search(cerca,uselike=False)
		except Exception,e:
			myerr = "%s" % e
			print myerr
			print "".join(traceback.format_tb(sys.exc_info()[2]))
			#raise e
			try:
				self.user = cerca # Sporco trucco: altrimenti i controlli isLoggedIn falliscono!!!
				self.initDB()
				newuser = self.dbmgr.getClazzByTypeName('DBEUser')(attrs={'login':user,'pwd':pwd})
				newuser = self.insert(newuser)
				self.user = newuser
				searchGroup = self.dbmgr.getClazzByTypeName('DBEGroup')(attrs={'name':user})
				newgroup = self.search(searchGroup,uselike=False)[0]
				if self.verbose: print "MongoConnectionProvider.login: newuser=%s" % ( newuser )
				newfolder = self.dbmgr.getClazzByTypeName('DBEFolder')(attrs={\
					'owner':newuser.getValue('id'),'group_id':newgroup.getValue('id'),\
					'creator':newuser.getValue('id'),'last_modify':newuser.getValue('id'),\
					'name':user})
				newfolder = self.insert(newfolder)
				if self.verbose: print "MongoConnectionProvider.login: newfolder=%s" % ( newfolder )
				ret = self.search(cerca,uselike=False)
			except Exception,e1:
				if self.verbose: print "MongoConnectionProvider.login: ECCEZIONE=%s" % ( e )
				if self.verbose: print "".join(traceback.format_tb(sys.exc_info()[2]))
		if len(ret)==1:
			self.user = ret[0]
		else:
			self.user = None
		if self.verbose: print "MongoConnectionProvider.login: self.user = %s" % ( self.user )
		if self.verbose: print "MongoConnectionProvider.login: end."
		return self.user
	def getLoggedUser(self):
		return self.user
	def getFormSchema(self,language='python',aClassname=''):
		#self.semaforo.acquire(True)
		myformschema, tmp = self._server.getFormSchema(language,aClassname)
		#self.semaforo.release()
		if self.verbose:
			print "myformschema:",myformschema
			print "tmp:",tmp
		return myformschema
	def getDBSchema(self,language='python',aClassname=''):
		#self.semaforo.acquire(True)
		myformschema, tmp = self._server.getDBSchema(language,aClassname)
		#self.semaforo.release()
		if self.verbose:
			print "myformschema:",myformschema
			print "tmp:",tmp
		return myformschema
	def objectById(self,myid,ignore_deleted=True):
		tipi = self.dbmgr.getRegisteredTypes()
		DBEObject = self.dbmgr.getClazzByTypeName("DBEObject")
		#DBAssociation = self.dbmgr.getClazzByTypeName("DBAssociation")
		q = []
		for tablename, clazz in tipi.items():
			mydbe = clazz()
			if not isinstance(mydbe, DBEObject):# or isinstance(mydbe, DBAssociation): # or mydbe.getTypeName=="DBEObject":
				continue
			tmp_q = "select '%s' as classname,id,owner,group_id,permissions,creator,creation_date,last_modify,last_modify_date,father_id,name,description from %s where id='%s'" % (mydbe.getTypeName(), self.dbmgr.buildTableName(mydbe), DBEObject.hex2uuid(myid) )
			if ignore_deleted:
				tmp_q += " and deleted_date='0000-00-00 00:00:00'"
			q.append( tmp_q )
		searchString = " union ".join( q )
		if self.verbose: print "ObjectMgr.objectById: searchString=%s" % ( searchString )
		raise DBLayerException("DA FINIRE")
		lista = self.select("objects", searchString)
		if len(lista)==1:
			return lista[0]
		if self.verbose: print "ObjectMgr.objectById: lista=%s" % ( len(lista) )
		return None

	def fullObjectById(self,myid,ignore_deleted=True):
		#self.semaforo.acquire(True)
		ret=[]
		try:
			self.msg, ret = self._server.fullObjectById(myid,ignore_deleted)
			self.msg = b64decode(self.msg)
		except Exception,e:
			print "MongoConnectionProvider.fullObjectById: ECCEZIONE ############# Start."
			print "ECCEZIONE: %s" % ( e )
			print "".join( traceback.format_tb(sys.exc_info()[2]) )
			print "MongoConnectionProvider.fullObjectById: ECCEZIONE ############# End."
		finally:
			#self.semaforo.release()
			pass
		if self.verbose:
			print "msg:",self.msg
			print "ret:",ret
		if len(ret)==1:
			return self._jsonToDbe(ret[0])
		return None
	def objectByName(self,myid,ignore_deleted=True):
		#self.semaforo.acquire(True)
		ret=[]
		tmp = self._server.objectByName(myid,ignore_deleted)
		if len(tmp)>0:
			self.msg = b64decode(tmp[0])
		if len(tmp)>1:
			ret=tmp[1:][0]
		#self.semaforo.release()
		if self.verbose:
			print "msg:",self.msg
			print "ret:",ret
		# 2012.05.07: start.
		return [self._jsonToDbe(x) for x in ret]
		#if len(ret)==1:
		#	return self._jsonToDbe(ret[0])
		#return None
		# 2012.05.07: end.
	def fullObjectByName(self,myid,ignore_deleted=True):
		#self.semaforo.acquire(True)
		ret=[]
		try:
			tmp = self._server.objectByName(myid,ignore_deleted)
			if len(tmp)>0:
				self.msg = b64decode(tmp[0])
			if len(tmp)>1:
				ret=tmp[1:][0]
			#self.msg, ret = self._server.fullObjectByName(myid,ignore_deleted)
			#self.msg = b64decode(self.msg)
		except Exception,e:
			print "MongoConnectionProvider.fullObjectByName: ECCEZIONE ############# Inizio."
			print "ECCEZIONE: %s" % ( e )
			print "".join( traceback.format_tb(sys.exc_info()[2]) )
			print "MongoConnectionProvider.fullObjectByName: ECCEZIONE ############# Fine."
			raise DBLayerException( "Communication troubles with the remote server: %s" % e )
		finally:
			#self.semaforo.release()
			pass
		if self.verbose:
			print "msg:",self.msg
			print "ret:",ret
		# 2012.05.07: start.
		return [self._jsonToDbe(x) for x in ret]
		#if len(ret)==1:
		#	return self._jsonToDbe(ret[0])
		#return None
		# 2012.05.07: end.
	def uploadFile(self,local_filename):
		# Remote destination directory
		#self.semaforo.acquire(True)
		messaggio, ret = self._server.GetRandomUploadDestinationDirectory()
		messaggio = b64decode(messaggio)
		#self.semaforo.release()
		if self.verbose:
			print "MongoConnectionProvider.uploadFile: messaggio = %s" % ( messaggio )
			print "MongoConnectionProvider.uploadFile: ret = %s" % ( ret )
		dest_dir = ret['dest_dir']
		if self.verbose:
			print "MongoConnectionProvider.uploadFile: dest_dir:",dest_dir
		# Short local filename
		filename_corto = local_filename[ len(os.path.dirname(local_filename)): ]
		if filename_corto.startswith('/'):
			filename_corto = filename_corto[1:]
		if self.verbose: print "MongoConnectionProvider.uploadFile: filename_corto: ", filename_corto
		# Load file
		myfile = file( local_filename ).read()
		# Upload TODO split the file in chunks
		#self.semaforo.acquire(True)
		messaggio, ret = self._server.Upload(1,1,filename_corto,u"%s" % Binary(myfile), dest_dir)
		messaggio = b64decode(messaggio)
		#self.semaforo.release()
		if self.verbose:
			print "MongoConnectionProvider.uploadFile: messaggio = %s" % ( messaggio )
			print "MongoConnectionProvider.uploadFile: ret = %s" % ( ret )
		filename = ret['filename']
		dest_dir = ret['dest_dir']
		if self.verbose:
			print "MongoConnectionProvider.uploadFile: filename: ", filename
			print "MongoConnectionProvider.uploadFile: dest_dir: ", dest_dir
		return "%s/%s" % (dest_dir,filename)
		#raise DBLayerException("MongoConnectionProvider.uploadFile: not implemented.")
	def downloadFile(self,remote_filename,local_path,view_thumbnail=False):
		uuid = remote_filename
		#self.semaforo.acquire(True)
		messaggio, ret = self._server.Download(uuid,view_thumbnail)
		messaggio = b64decode(messaggio)
		#self.semaforo.release()
		if self.verbose:
			print "MongoConnectionProvider.downloadFile: messaggio = %s" % ( messaggio )
			tmp = {}
			for k in ret.keys():
				if k=='contents': continue
				tmp[k]=ret[k]
			print "MongoConnectionProvider.downloadFile: ret (senza contents) = %s" % ( tmp )
		mime = "%s" % ret['mime']
		filesize = ret['filesize']
		filename = "%s%s%s" % (local_path,os.path.sep,ret['filename'])
		if self.verbose:
			print "MongoConnectionProvider.downloadFile: mime = %s" % ( mime )
			print "MongoConnectionProvider.downloadFile: filesize = %s" % ( filesize )
			print "MongoConnectionProvider.downloadFile: filename = %s" % ( filename )
		file(filename,'wb').write( b64decode(ret['contents']) )
		return filename





# -*- coding: utf-8 -*-

#
# @copyright &copy; 2012 by Roberto Rocco Angeloni <roberto@roccoangeloni.it>
# @license http://opensource.org/licenses/lgpl-3.0.html GNU Lesser General Public License, version 3.0 (LGPLv3)
# @version $Id: editwidgetcontroller.py $
# @package qtwidgets
# 
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

"""
Server Widget

Includes a single ExplorerWidget with a stacked form widgets

For a single server connection only.

"""

import os,sys,traceback
from PyQt4 import QtCore, QtGui, QtWebKit

try:
	_fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
	_fromUtf8 = lambda s: s

class WebWidget(QtGui.QWidget):
	def __init__(self,parent,objectName, url_homepage):
		QtGui.QWidget.__init__(self,parent)
		self.url_homepage = url_homepage
		w = self
		w.setAutoFillBackground(False)
		w.setObjectName("w_%s"%objectName)
		self.gridLayout = QtGui.QGridLayout(w)
		self.gridLayout.setObjectName("gridLayout_%s"%objectName)
		self.gridLayout.setMargin(0)
		self.webAddress = QtGui.QLineEdit(w)
		self.webAddress.setObjectName("webAddress_%s"%objectName)
		self.gridLayout.addWidget(self.webAddress, 0, 2, 1, 1)
		self.frame = QtGui.QFrame(w)
		self.frame.setFrameShape(QtGui.QFrame.StyledPanel)
		self.frame.setFrameShadow(QtGui.QFrame.Sunken)
		self.frame.setObjectName("frame_%s"%objectName)
		self.verticalLayout = QtGui.QVBoxLayout(self.frame)
		self.verticalLayout.setMargin(0)
		self.verticalLayout.setObjectName("verticalLayout")
		self.webView = QtWebKit.QWebView(self.frame)
		self.webView.setAutoFillBackground(False)
		self.webView.setUrl(QtCore.QUrl(self.url_homepage))
		self.webView.setObjectName("webView_%s"%objectName)
		self.verticalLayout.addWidget(self.webView)
		self.gridLayout.addWidget(self.frame, 1, 0, 1, 3)
		self.label = QtGui.QLabel(w)
		self.label.setObjectName("label_%s"%objectName)
		self.label.setText(QtGui.QApplication.translate("WebComponent", "Url:", None, QtGui.QApplication.UnicodeUTF8))
		self.gridLayout.addWidget(self.label, 0, 1, 1, 1)
		# Slots
		self.bind()
		# Logic
		self.server = None
		self.formFactory = None
		# Container
		self.container = None
	def bind(self):
		QtCore.QObject.connect(self.webAddress,QtCore.SIGNAL("returnPressed()"),self.webAddress_returnPressed)
		QtCore.QObject.connect(self.webView,QtCore.SIGNAL("loadProgress(int)"),self.loadProgress)
		QtCore.QObject.connect(self.webView,QtCore.SIGNAL("loadFinished(bool)"),self.loadFinished)
		QtCore.QObject.connect(self.webView,QtCore.SIGNAL("urlChanged (const QUrl&)"),self.urlChanged)
		QtCore.QObject.connect(self.webView,QtCore.SIGNAL("statusBarMessage (const QString&)"),self.SetStatusText)
# ############################## PROPERTIES ###################################
	def setContainer(self,c):
		self.container = c
	def setServer(self,server):
		self.server = server
	def getServer(self):
		return self.server
	def setFormFactory(self,formFactory):
		self.formFactory = formFactory
		# Update UI
		print "WebWidget.setFormFactory: TODO"
	def getName(self):
		return self.server.getServerIDString()
# ############################## PREFERENCES ###################################
	def getPref(self,section,option,default=None):
		if self.container is None:
			return default
		return self.container.getPref(section,option,default)
	def setPref(self,section,option,valore):
		if self.container is None:
			return
		self.container.setPref(section,option,valore)
# ############################## ... ###################################
	def Alert(self,msg, eccezione=None):
		if self.container is None:
			return
		self.container.Alert(msg,eccezione)
	def Confirm(self,msg):
		if self.container is None:
			return True
		return self.container.Confirm(msg)
	def SetStatusText(self,msg):
		if self.container is None:
			return
		self.container.SetStatusText(msg)
	def SetProgress(self,perc):
		if self.container is None:
			return
		self.container.SetProgress(perc)
# ############################## UI ###################################
# ############################## LOGIC ###################################
# ############################## SLOTS ###################################
	def urlChanged(self,s):
		self.webAddress.setText(s.toString())
	def loadProgress(self,x):
		self.SetStatusText("Loaded %d%%"%x)
		self.SetProgress(x)
	def loadFinished(self,b):
		if b:
			self.SetStatusText("")
		else:
			self.SetStatusText("Error while loading.")
	def hideNavigation(self):
		self.webAddress.hide()
		self.label.hide()
	def showNavigation(self):
		self.webAddress.show()
		self.label.show()
	def webAddress_returnPressed(self):
		if ("%s"%self.webAddress.text()).startswith("http://"):
			self.webView.setUrl(QtCore.QUrl(self.webAddress.text()))
		else:
			self.webView.setUrl(QtCore.QUrl("http://%s" % self.webAddress.text()))



#
# @copyright &copy; 2011 by Roberto Rocco Angeloni <roberto@roccoangeloni.it>
# @license http://opensource.org/licenses/lgpl-3.0.html GNU Lesser General Public License, version 3.0 (LGPLv3)
# @version $Id: winodbc.py $
# @package dblayer
# 
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

# -*- coding: utf-8 -*-

import os,sys

# Path to RRA python framework: start.
mypath = os.path.realpath( os.path.dirname(os.path.realpath(__file__)) +'/../../../python')
if not os.path.exists( mypath):
	mypath = os.path.realpath( os.path.dirname(os.path.realpath(__file__)) +'/../../python')
if not mypath in sys.path:
	sys.path.append(mypath)
# Required to redirect outout to a file
import codecs
sys.stdout = codecs.lookup('utf8')[-1](sys.stdout)   # 4
# Path to RRA python framework: end.


from rprj.dblayer import *

import odbc

class WinOdbcConnectionProvider(DBConnectionProvider):
	"""WinOdbc Connection provider"""
	
	__super_init = DBConnectionProvider.__init__
	
	def __init__(self, dsn, host=None, db=None, user=None, pwd=None, verbose=1):
		self.__super_init( host, db, user, pwd, verbose )
		self.dsn=dsn
	
	def getConnection(self):
		if self._conn is None:
			try:
				self._conn = odbc.odbc( self.dsn )
			except Exception, e:
				if self._verbose: print "WinOdbcConnectionProvider.getConnection: Error=%s" % (e)
		return self._conn
	
	def freeConnection(self,conn):
		try:
			conn.close()
		except Exception, e:
			if self._verbose: print "WinOdbcConnectionProvider.freeConnection: Error=%s" % (e)
		del conn
		del self._conn
		self._conn = None
	
	def getDBType(self):
		return "WinODBC"
	
	def _description2names(self, desc):
		"""Converte la description del resultset fornita dalle api DB
		in un array di nomi"""
		ret = []
		for d in desc:
			ret.append( d[0] )
		return ret
	def filterAccessName(self, accessName):
		return accessName.replace("[","").replace("]","").replace(" ","") #.replace(" ","_")
	def description2sql(self,desc,debug=False):
		"""(name, type_code, display_size, internal_size, precision, scale, null_ok)"""
		if debug: print "(name, type_code, display_size, internal_size, precision, scale, null_ok)"
		if debug: print desc
		ret=[]
		for d in desc:
			if debug: print d
			name, type_code, display_size, internal_size, precision, scale, null_ok = d
			tmp=["",""]
			tmp.append(name)
			if type_code=='STRING':
				if display_size==internal_size:
					tmp.append("varchar(%s)"%internal_size)
				elif display_size==0 and internal_size>0:
					tmp.append("text")
			elif type_code=='NUMBER':
				tmp.append("int(%s)"%internal_size)
			elif type_code=='DATE':
				tmp.append("char(%s)"%internal_size)
			else:
				tmp.append("%s(%s,%s,%s,%s)" % (type_code, display_size, internal_size, precision, scale))
			if not null_ok:
				tmp.append("not null")
			ret.append( " ".join(tmp) )
		return ",\n".join(ret)
	def table2sql(self,tablename,keys=[],debug=False):
		_tablename = self.filterAccessName(tablename)
		cursor = self.getConnection().cursor()
		cursor.execute( "select * from %s" % tablename )
		names = None
		ret = ["create table %s ( " % _tablename]
		try:
			names = self._description2names(cursor.description)
			ret.append("%s," % self.description2sql(cursor.description,debug))
		except TypeError,e:
			if self._verbose: print "DBMgr.select: ECCEZIONE=%s (%s)" % (e,searchString)
			pass
		cursor.close()
		ret.append("  primary key (%s)" % (",".join(keys)) )
		ret.append(");")
		return "\n".join(ret)
	def dumpData(self,tablename, debug=False):
		_tablename = self.filterAccessName(tablename)
		cursor = self.getConnection().cursor()
		cursor.execute( "select * from %s" % tablename )
		names = None
		ret = []
		try:
			names = self._description2names(cursor.description)
			listavalori = cursor.fetchall()
			numres = len(listavalori)
			for i in range( numres ):
				riga=[u"insert into %s values (" % _tablename]
				tmp=[]
				for n in range(0,len(names)):
					if listavalori[ i ][n] is None:
						tmp.append("NULL")
						continue
					type_code=cursor.description[n][1]
					valore = "%s"%listavalori[ i ][n]
					valore = valore.replace(u'\u2013','-')
					valore = valore.replace(u'\u2019','\'')
					valore = valore.replace(u'\u2026',"...")
					valore = valore.replace(u'\u20ac',"<Euro>")
					valore = valore.replace('\'','\'\'')
					valore = valore.replace('\r\n','\\r\\n')
					valore = valore.replace('\n','\\n')
					if type_code=='STRING' or type_code=='DATE':
						tmp.append( u"'%s'" % valore )
					else:
						tmp.append( u"%s" % valore )
				riga.append(u",".join(tmp))
				riga.append( u")" )
				try:
					#print "".join(riga)
					ret.append("".join(riga))
				except Exception,e:
					if debug:
						for n in range(0,len(names)):
							valore = "%s"%listavalori[ i ][n]
							valore = valore.replace(u'\u2013','-')
							valore = valore.replace(u'\u2019','\'')
							valore = valore.replace(u'\u2026',"...")
							valore = valore.replace(u'\u20ac','<Euro>')
							print names[n],valore
		except TypeError,e:
			if self._verbose: print "DBMgr.select: ECCEZIONE=%s" % (e)
			pass
		cursor.close()
		return u";\n".join(ret)

	def execute(self,searchString):
		"""Returns a list of DBE according to the searchString"""
		if self._verbose: print "DBMgr.select: searchString=%s" % ( searchString )
		cursor = self.getConnection().cursor()
		cursor.execute( searchString )
		#numres = cursor.rowcount
		names = None
		ret = []
		try:
			#print "cursor:",dir(cursor)
			#print "description:",cursor.description
			names = self._description2names(cursor.description)
			listavalori = cursor.fetchall()
			numres = len(listavalori)
			if self._verbose: print "DBMgr.select: found %s rows." % ( numres )
			for i in range( numres ):
				tmp={}
				for n in range(0,len(names)):
					#if isDateTime(values[i]): # values[i]=string2datetime(values[i])
						#setattr( self, names[i] , string2datetime(values[ i ]) )
					#else:
					tmp[names[n]] = listavalori[ i ][n]
				ret.append( tmp )
		except TypeError,e:
			if self._verbose: print "DBMgr.select: ECCEZIONE=%s (%s)" % (e,searchString)
			pass
		cursor.close()
		return ret



# -*- coding: utf-8 -*-

#
# @copyright &copy; 2012 by Roberto Rocco Angeloni <roberto@roccoangeloni.it>
# @license http://opensource.org/licenses/lgpl-3.0.html GNU Lesser General Public License, version 3.0 (LGPLv3)
# @version $Id: editwidgetcontroller.py $
# @package qtwidgets
# 
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

"""

Signals:
updatedList()
selected(QListWidgetItem,PyQt_PyObject)

"""
import datetime,os,sys,traceback
from PyQt4 import QtCore,QtGui

try:
	_fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
	_fromUtf8 = lambda s: s

DEBUG_WIDGET=True

class SearchWidgetItemDelegate(QtGui.QItemDelegate):
	# http://doc.qt.digia.com/qt/model-view-programming.html
	# http://doc.qt.digia.com/qt/qabstractitemview.html#setItemDelegate
	# QItemDelegate(QObject parent=None)
	def __init__(self,myfield, parent=None):
		QtGui.QItemDelegate.__init__(self,parent)
		#self.myeditorclazz = myeditorclazz
		self.myfield = myfield
	
	# QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const;
	def createEditor(self, parent, option, index):
		#self.myinstance = self.myeditorclazz()
		editor = self.myfield.InitField(parent)
		#editor.setMinimum(0)
		#editor.setMaximum(100)
		return editor
	
	# void setEditorData(QWidget *editor, const QModelIndex &index) const;
	def setEditorData(self, editor, index):
		value = index.model().data(index, QtCore.EditRole).toInt()
		#QSpinBox *spinBox = static_cast<QSpinBox*>(editor)
		spinBox = editor
		spinBox.setValue(value)

	# void setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const;
	def setModelData(self, editor, model, index):
		value = editor.getValue()
		model.setData(index, value, QtCore.EditRole) # Qt.EditRole
	
	# void updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &index) const;
	def updateEditorGeometry(self, editor, option, modelindex):
		editor.field.setGeometry(option.rect)

class SearchWidget(QtGui.QWidget):
	def __init__(self,parent,objectName,filterForm,modifyFilter=True):
		QtGui.QWidget.__init__(self,parent)
		# UI
		self.setObjectName(objectName)
		self.resize(360, 360)
		self.setWindowTitle(QtGui.QApplication.translate("RprjSearchWidget", "Search", None, QtGui.QApplication.UnicodeUTF8))
		self.verticalLayout = QtGui.QVBoxLayout(self)
		self.verticalLayout.setMargin(0)
		self.verticalLayout.setObjectName("%s_verticalLayout"%objectName)
		# Filter
		self.widgetFilter = filterForm.render_filter(self)
		self.verticalLayout.addWidget(self.widgetFilter,0)
		# Button Search
		icon = QtGui.QIcon()
		icon.addPixmap(QtGui.QPixmap(":/rPrj/icons/reload.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
		self.buttonSearch = QtGui.QPushButton(icon,"",self)
		self.buttonSearch.setToolTip("Search")
		self.buttonSearch.setShortcut("Ctrl+R")
		QtCore.QObject.connect(self.buttonSearch,QtCore.SIGNAL("clicked()"),self.slotDoSearch)
		self.verticalLayout.addWidget(self.buttonSearch,0)
		self.filterForm = filterForm
		# List
		self.widgetList = QtGui.QTableView(self)
		self.widgetList.setSortingEnabled(True)
		self.columnNames = self.filterForm.getListColumnNames()
		self.model = QtGui.QStandardItemModel(0, len(self.columnNames)+1, self.widgetList)
		self.model.setHeaderData(0, QtCore.Qt.Horizontal, QtCore.QVariant( "Type" ))
		for i in range(len(self.columnNames)):
			self.model.setHeaderData(i+1, QtCore.Qt.Horizontal, QtCore.QVariant( self.filterForm.getField( self.columnNames[i] ).GetTitle() ))
			if self.columnNames[i]=='stato':
				myfieldinstance = self.filterForm.getField( self.columnNames[i] )
				print "myfieldinstance:",myfieldinstance
				myitemdelegate = SearchWidgetItemDelegate(myfieldinstance)
				self.widgetList.setItemDelegateForColumn( i+1, myitemdelegate )
			# TEST
			#if self.columnNames[i]=='stato':
				#myitemdelegate = SearchWidgetItemDelegate(myfield)
				#self.widgetList.setItemDelegateForColumn( i+1, myitemdelegate )
		self.widgetList.setModel( self.model )
		self.verticalLayout.addWidget(self.widgetList,1)
		# Slots
		self.bind()
		# Logic
		self.server = None
		self.formFactory = None
		self.setModifyFilter(modifyFilter)
		self._selectedDBE = None
		# Container
		self.container = None
		# DragDrop
		self.setAcceptDrops(True)
	def bind(self):
		# Search
		QtCore.QObject.connect(self.widgetFilter,QtCore.SIGNAL("doSearch()"),self.slotDoSearch)
		# List Widget
		QtCore.QObject.connect(self.widgetList,QtCore.SIGNAL("clicked(QModelIndex*)"),self.slotItemClicked)
		QtCore.QObject.connect(self.widgetList,QtCore.SIGNAL("doubleClicked(QModelIndex*)"),self.slotItemDoubleClicked)

# ############################## PROPERTIES ###################################
	def setServer(self,server):
		self.server = server
	def getServer(self):
		return self.server
	def setFormFactory(self,formFactory):
		self.formFactory = formFactory
		# Update UI
		pass
	def getName(self):
		d = self.server.getConnectionProvider().getDBType()
		u = "nobody"
		if not self.server.getDBEUser() is None:
			u = self.server.getDBEUser().getValue('login')
		h = "%s" % self.server.getConnectionProvider().getHost()
		if d=="SQLite":
			h = ( "%s" % self.server.getConnectionProvider().getDB() ).split( os.path.sep )[-1]
		if h.find("://")>0:
			_url = h.split("://")
			if len(_url)>1:
				h = "%s:%s" % ( _url[0],_url[1].split("/")[0] )
		return "%s@%s" % (u,h)
	def setModifyFilter(self,b):
		self._modifyFilter = b
		self.widgetFilter.setVisible( b )
		self.buttonSearch.setVisible( not b )
	def modifyFilter(self):
		return self._modifyFilter
	def setSelectedDBE(self,dbe):
		self._selectedDBE = dbe
	def selectedDBE(self):
		return self._selectedDBE
	def setValues(self, values):
		self.filterForm.setValues(values)
	def getValues(self, values):
		return self.filterForm.getValues()
	def cleanValues(self):
		return self.filterForm.clean()
# ############################## PREFERENCES ###################################
	def getPref(self,section,option,default=None):
		if self.container is None:
			return default
		return self.container.getPref(section,option,default)
	def setPref(self,section,option,valore):
		if self.container is None:
			return
		self.container.setPref(section,option,valore)
# ############################## ... ###################################
	def Alert(self,msg, eccezione=None):
		if self.container is None:
			return
		self.container.Alert(msg,eccezione)
	def Confirm(self,msg):
		if self.container is None:
			return True
		return self.container.Confirm(msg)
	def SetStatusText(self,msg):
		if self.container is None:
			return
		self.container.SetStatusText(msg)
	def SetProgress(self,perc):
		if self.container is None:
			return
		self.container.SetProgress(perc)
# ############################## UI ###################################
	def updateList(self,lista):
		self.model.removeRows(0, self.model.rowCount())
		for l in lista:
			_mytypename = l.getTypeName()
			if _mytypename=='DBEObject':
				_mytypename=l.getValue('classname')
			myform = self.formFactory.getInstanceByDBEName( _mytypename, dbmgr=self.server)
			myform.setValues( l.getValuesDictionary() )
			row = self.model.rowCount()
			self.model.insertRow(row)
			self._formToRow(row,myform)
		self.widgetList.resizeColumnsToContents()
		self.widgetList.resizeRowsToContents()
		self.emit(QtCore.SIGNAL("updatedList()"))
	def _formToRow(self,row,myform):
		readOnlyColumnNames = myform.getDetailReadOnlyColumnNames()
		icon = QtGui.QIcon()
		icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/rPrj/%s"%myform.getDetailIcon())), QtGui.QIcon.Normal, QtGui.QIcon.Off)
		self.model.setData(self.model.index(row, 0), u"%s"%myform.getDetailTitle() )
		self.model.item(row,0).setEditable(False)
		self.model.item(row,0).setIcon( icon )
		for i in range(len(self.columnNames)):
			myfield = myform.getField(self.columnNames[i])
			myvalue = "%s"%myfield.getValue()
			self.model.setData(self.model.index(row, i+1), myvalue )
			self.model.setData(self.model.index(row, i+1), QtCore.QVariant(row), 32 )
			if self.columnNames[i] in readOnlyColumnNames:
				self.model.item(row,i+1).setEditable(False)
			else:
				self.model.item(row,i+1).setEditable(True)
# ############################## LOGIC ###################################
	def runSearch(self,searchdbe):
		"""A better name..."""
		self.SetProgress(33)
		lista = self.server.search(searchdbe)
		self.SetProgress(66)
		self.updateList(lista)
		self.SetProgress(100)
# ############################## SLOTS ###################################
	def slotDoSearch(self):
		cerca = self.filterForm.getDBE(self.server)
		tmp_values = self.filterForm.getValues()
		searchValues = {}
		for k,v in tmp_values.items():
			myv=v
			if isinstance(myv,datetime.datetime):
				myv = self.server.datetime2string(myv)
			elif isinstance(myv,QtCore.QString):
				myv = "%s" % v
			else:
				myv = "%s" % v
			if myv.find('0000-00-00 00:00')>=0: continue
			searchValues[k]=myv
		cerca.setValuesDictionary( searchValues )
		self.runSearch(cerca)
	def slotItemClicked(self,item):
		pass
	def slotItemDoubleClicked(self,item):
		mydata = item.data(QtCore.Qt.UserRole).toPyObject()
		self.setSelectedDBE(mydata)
		self.emit(QtCore.SIGNAL("selected(QListWidgetItem,PyQt_PyObject)"),item,mydata)



# -*- coding: utf-8 -*-

#
# @copyright &copy; 2012 by Roberto Rocco Angeloni <roberto@roccoangeloni.it>
# @license http://opensource.org/licenses/lgpl-3.0.html GNU Lesser General Public License, version 3.0 (LGPLv3)
# @version $Id: editwidgetcontroller.py $
# @package qtwidgets
# 
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

"""
R-Project webbrowser component

"""

import os,sys,traceback
from PyQt4 import QtCore, QtGui

try:
	_fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
	_fromUtf8 = lambda s: s

from qtcomponent import *
from rprj.formulator import apps_rc,  webwidget

class WebComponent(qtComponent):
	"""Web Component di Esempio"""
	def __init__(self, config, parent=None):
		qtComponent.__init__(self, config, parent)
		self._objectNamePrefix="rprj_web"
		self.widget = None
		self.menu = None
		self.url_homepage = self.getPref( self.__class__.__name__ , "url_homepage", "http://www.kware.it/main.php?obj_id=uuid69742e34663137643339336236653937" )
		self.server = None
		self.formFactory = None
	def init(self):
		self.w=None
		self.connections = []
	def renderWidget(self,parentWidget):
		self.widget = QtGui.QTabWidget(parentWidget)
		self.widget.setTabsClosable(True)
		self.widget.setMovable(True)
		self.widget.setObjectName("%s_widget"%self._objectNamePrefix)
		# Binds
		# Tabs
		QtCore.QObject.connect(self.widget,QtCore.SIGNAL("tabCloseRequested(int )"),self.slotTabCloseRequested)
	def renderMenu(self,menubar):
		# Main Menu
		self.menu = QtGui.QMenu(menubar)
		self.menu.setTitle(QtGui.QApplication.translate("Rprj", "Web", None, QtGui.QApplication.UnicodeUTF8))
		self.menu.setObjectName("%s_menu"%self._objectNamePrefix)
		# Menu Admin
		if self.getPref("Main","Admin",False):
			self.menuAdmin = QtGui.QMenu(self.menu)
			self.menuAdmin.setTitle(QtGui.QApplication.translate("Rprj", "Admin", None, QtGui.QApplication.UnicodeUTF8))
			self.menuAdmin.setObjectName("%s_menuAdmin"%self._objectNamePrefix)
			#self.actionCheckDB = QtGui.QAction(self.menuAdmin)
			#self.actionCheckDB.setText(QtGui.QApplication.translate("Rprj", "Check DB", None, QtGui.QApplication.UnicodeUTF8))
			#self.actionCheckDB.setObjectName(_fromUtf8("actionCheckDB"))
			#self.menuAdmin.addAction(self.actionCheckDB)
			pass
		# Only Admins should see this
		if self.getPref("Main","Admin",False):
			self.menu.addSeparator()
			self.menu.addMenu(self.menuAdmin)
		menubar.addAction(self.menu.menuAction())
		# Binds
		# Menu
		#QtCore.QObject.connect(self.actionCheckDB,QtCore.SIGNAL("triggered()"),self.slotCheckDB)
		# Admin Menu
		if self.getPref("Main","Admin",False):
			#QtCore.QObject.connect(self.actionCheckDB,QtCore.SIGNAL("triggered()"),self.slotCheckDB)
			pass
# ############################## DESCRIPTION ###################################
	def getIcon(self):
		return QtGui.QIcon(":/rPrj/icons/linneighborhood.png")
	def getName(self):
		return "Web"
	def getPanel(self,parent=None):
		"""Returns a QtGui.QWidget"""
		if self.widget is None:
			self.renderWidget(parent)
		return self.widget
	def getMenu(self,menubar):
		if self.menu is None:
			self.renderMenu(menubar)
		return self.menu
# ############################## PREFERENCES ###################################
	def SaveUI2Config(self):
		pass
	def RestoreUIFromConfig(self):
		pass
# ############################## LOGIC ###################################
	def connectToServer(self, server,  formFactory):
		self.server = server
		self.formFactory = formFactory
		# Exists?
		serverIDstring = server.getServerIDString()
		for c in self.connections:
			if c['name']!=serverIDstring:
				continue
			widget = c['widget']
			self.widget.setCurrentWidget( widget )
			return ""
		try:
			# Widget
			widget = webwidget.WebWidget(self.widget, "WebWidget", self.url_homepage)
			print "WebComponent.connectToServer: TODO"
			widget.setServer(server)
			widget.setFormFactory(formFactory)
			widget.setContainer(self)
			# Slots
			#QtCore.QObject.connect(widget,QtCore.SIGNAL("updatedTree()"),self.slotUpdatedTree)
			#QtCore.QObject.connect(widget,QtCore.SIGNAL("selected(QStandardItem,PyQt_PyObject)"),self.slotSelected)
			# tabBar
			if self.widget.count()==0:
				self.widget.tabBar().hide()
			else:
				self.widget.tabBar().show()
			# Add
			self.widget.addTab(widget, widget.getName() )
			self.widget.setCurrentWidget( widget )
			self.connections.append( { 'name':serverIDstring, 'widget':widget } )
		except Exception,e:
			print "ECCEZIONE:",e
			print "".join(traceback.format_tb(sys.exc_info()[2]))
		print "WebComponent.connectToServer: TODO"
# ############################## SLOTS ###################################
	def slotTabCloseRequested(self,index):
		if not self.Confirm("Close this connection?"):
			return
		serverwidget = self.widget.widget( index )
		self.widget.removeTab( index )
		# Exists?
		for i in range(len(self.connections)):
			if self.connections[i]['widget']!=serverwidget:
				continue
			self.connections.pop(i)
			break
		del serverwidget



# -*- coding: utf-8 -*-

#
# @copyright &copy; 2012 by Roberto Rocco Angeloni <roberto@roccoangeloni.it>
# @license http://opensource.org/licenses/lgpl-3.0.html GNU Lesser General Public License, version 3.0 (LGPLv3)
# @version $Id: __init__.py $
# @package formulator
# 
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

class FField:
	def __init__(self, aNomeCampo, aTitle, aDescription, aSize, aLength=20, aValore='', aClasseCss=None, myform=None, mytipo='s' ):
		self.aNomeCampo = aNomeCampo
		self._title = aTitle
		self._description = aDescription
		self._size = aSize
		self._length = aLength
		self.aValore = aValore
		self.aClasseCss = aClasseCss
		self.myform = myform
		self.tipo = mytipo
	def render(self):
		raise Exception("TODO")
	def render_view(self):
		raise Exception("TODO")
	def render_hidden(self):
		raise Exception("TODO")
	def render_readonly(self):
		raise Exception("TODO")
	def readValueFromRequest(self, request, prefix='field_'):
		self.readValueFromArray(request,prefix)
	def readValueFromArray(self, aArray, prefix='field_'):
		if aArray.has_key("%s%s" % (prefix,self.aNomeCampo)):
			self.aValore = aArray[ "%s%s" % (prefix,self.aNomeCampo) ]
		else:
			self.aValore = None
	def writeValueToArray(self, aArray, prefix='field_'):
		aArray["%s%s" % (prefix,self.aNomeCampo)] = self.aValore
		return aArray
	def clean(self):
		"""Resets the field value"""
		self.setValue('')
	def getValue(self):
		return self.aValore
	def setValue(self,v):
		if v=='':
			self.aValore=None
		else:
			self.aValore = v
	def GetTitle(self):
		return self._title
	def GetDescription(self):
		return self._description

class FForm:
	def __init__(self, nome='', azione='', metodo="POST", enctype='',dbmgr=None,formFactory=None):
		self.fields = {}
		self.groups = {}
		self.nome = nome
		self.azione = azione
		self.metodo = metodo
		self.enctype = enctype
		self.dbmgr=dbmgr
		self.formFactory=formFactory
	def toString(self):
		ret = "Nome: %s\n" % self.nome
		ret += "Azione: %s\n" % self.azione
		ret += "Metodo: %s\n" % self.metodo
		gruppi = self.getGroupNames()
		ret += "Gruppi: %s\n" % gruppi
		for nomeGruppo in gruppi:
			ret += "\tGruppo: %s\n" % nomeGruppo
			mygruppo = self.getGroup( nomeGruppo )
			for i in range(len(mygruppo)):
			#for i in range(mygruppo):
				myfield = self.getField( mygruppo[ i ] )
				ret += "\t\t%s => '%s'\n" % (mygruppo[i],myfield.getValue())
		return ret
	def getAction(self):
		"""Azione di default: serve per la form html"""
		return self.azione
	def getMethod(self):
		return self.metodo
	def getName(self):
		return self.nome
	def getEnctype(self):
		return self.enctype
	# OVERRIDE: start.
	def getDetailIcon(self):
		return ""
	def getDetailTitle(self):
		return ""
	def getDetailColumnNames(self):
		return []
	def getDetailReadOnlyColumnNames(self):
		"""Ritorna i nomi dei campi in read-only"""
		return []
	def getFilterForm(self):
		return None
	def getFilterFields(self):
		return []
	def getListTitle(self):
		return ""
	def getListColumnNames(self):
		"""Ritorna i nomi di fields da visualizzare in una lista"""
		return []
	def getListEditableColumnNames(self):
		"""Ritorna i nomi di fields editabili in lista"""
		return []
	def getDecodeGroupNames(self):
		"""Ritorna un array con nome_gruppo=>'Nome Gruppo'"""
		return {}
	def getPagePrefix(self):
		return ""
	def getDBE(self):
		"""Ritorna la dbe associata alla form"""
		return None
	def getCodice(self):
		"""Ritorna il codice che identifica la descrizione."""
		return ""
	def getShortDescription(self, dbmgr=None):
		"""Ritorna una breve descrizione dei valori contenuti."""
		return ""
	def getController(self):
		"""Ritorna il nome del controller della form"""
		return ""
	def getActions(self):
		"""Ritorna le azioni possibili sul tipo di dato gestito.
		L'array tornato e' del tipo: codice_azione => array('Label azione','immagineAzione.jpg')
		Le azioni saranno date in pasto assieme ai parametri modificati ad un controller."""
		return {}
	# OVERRIDE: end.
	# Fields: start.
	def addField( self, nomeGruppo, ordine, nomeField, aField ):
		aField.myform = self
		self.fields[ nomeField ] = aField
		self.addToGroup( nomeField, nomeGruppo, ordine )
		if isinstance(aField,FFileField):
			self.enctype='multipart/form-data'
	def getFieldNames(self):
		return self.fields.keys()
	def getField( self, fieldName ):
		if not self.fields.has_key(fieldName):
			return None
		if self.fields[ fieldName ].myform is None:
			self.fields[ fieldName ].myform = self
		return self.fields[ fieldName ]
	# Fields: end.
	# Groups: start.
	def decodeGroupName(self, group_name):
		tmp = self.getDecodeGroupNames()
		if tmp.has_key(group_name):
			return tmp[group_name]
		else:
			return group_name
	def getGroupNames(self):
		return self.groups.keys()
	def getGroup( self, nomeGruppo ):
		if self.groups.has_key(nomeGruppo):
			return self.groups[ nomeGruppo ]
		else:
			return []
	def getGroupSize( self, nomeGruppo ):
		return len( self.getGroup( nomeGruppo ) )
	def setGroup( self, nomeGruppo, gruppo ):
		self.groups[ nomeGruppo ] = gruppo
	def addToGroup( self, nomeField, nomeGruppo='', ordine=-1 ):
		"""Un gruppo e' un array ordinato di nomi di campo."""
		gruppo = self.getGroup( nomeGruppo )
		if ordine<0:	ordine = len( gruppo )
		if ordine>=len(gruppo):
			gruppo.append( nomeField )
		else:
			gruppo.insert( ordine, nomeField )
			#gruppo[ ordine ] = nomeField
		self.setGroup( nomeGruppo, gruppo )
	def removeFromGroup(self, nomeField, nomeGruppo):
		gruppo = self.getGroup( nomeGruppo )
		i = gruppo.index(nomeField)
		if i>=0:
			gruppo.pop( i )
		self.setGroup( nomeGruppo, gruppo )
	# Groups: end.
	# Request: start.
	def readValuesFromRequest(self,aRequest,prefix="field_"):
		for nomeCampo in self.getFieldNames():
			self.fields[ nomeCampo ].readValueFromRequest(aRequest,prefix)
	def readValuesFromArray(self, aSession, prefix="field_"):
		self.readValuesFromRequest(aSession,prefix)
	def writeValuesToArray(self, aSession, prefix="field_"):
		for nomeCampo in self.getFieldNames():
			self.fields[ nomeCampo ].writeValueToArray(aSession,prefix)
	# Request: end.
	# Getters and setters: start.
	def clean(self):
		"""Resets all the field values"""
		for n in self.getFieldNames():
			self.fields[n].clean()
	def getValues(self):
		ret = {}
		for nomeCampo in self.getFieldNames():
			if self.fields.has_key(nomeCampo): # 2012.04.17
				ret[ nomeCampo ] = self.fields[ nomeCampo ].getValue()
		return ret
	def setValues( self, valori ):
		for chiave in valori.keys():
			campo = self.getField(chiave)
			if not campo is None:
				self.fields[ chiave ].setValue( valori[ chiave ] )
	def getValue(self,fieldName):
		tmp = self.getField(fieldName)
		if tmp is None:
			return None
		return tmp.getValue()
	def setValue(self,fieldName,aValue):
		self.getField( fieldName ).setValue(aValue)
	def setFormFactory(self, formFactory):
		self.formFactory = formFactory
	def getFormFactory(self):
		return self.formFactory
	# Getters and setters: end.
	# Render: start.
	def render(self,dbmgr=None):
		raise Exception("TODO")
	def render_view(self,dbmgr=None):
		raise Exception("TODO")
	def render_filter(self,dbmgr=None):
		raise Exception("TODO")
	# Render: end.

class FMasterDetail(FForm):
	def __init__(self,nome='', azione='', metodo="POST",enctype='',dbmgr=None):
		FForm.__init__(self,nome,azione,metodo,enctype,dbmgr)
		self.detailForms=[]
		# Contenitore delle dbe figlie della corrente: 'nomeform'=>lista dbe figli
		self.childs={}
	def addDetail( self, aDetail, cardinality="n" ):
		"""Aggiunge una form di dettaglio
		@param aDetail istanza di form OPPURE stringa col nome della classe FForm da istanziare
		@param cardinality 1=solo un figlio n=n-figli"""
		self.detailForms.append(aDetail)
	def getDetailFormsCount(self):
		return len(self.detailForms)
	def getDetailName(self,i):
		return self.detailForms[i]
	def getDetail(self,i):
		if type(self.detailForms[i])==str:
			ret = (self.getFormFactory().getInstance(self.detailForms[i],dbmgr=self.dbmgr))
			return ret
		else:
			return self.detailForms[i]
	def setValues( self, valori, doFetchChilds=False ):
		FForm.setValues( self, valori )
		if doFetchChilds:
			self.fetchChilds()
	def fetchChilds(self):
		print "FMasterDetail.fetchChilds: self.dbmgr=%s" % self.dbmgr
		mydbe = self.getDBE(self.dbmgr)
		print "FMasterDetail.fetchChilds: %s - mydbe=%s" % ( self.__class__.__name__, mydbe )
		if mydbe is None:
			return
		mydbe.setValuesDictionary( self.getValues() )
		print "FMasterDetail.fetchChilds: mydbe=%s" % ( mydbe )
		for detail in self.detailForms:
			print "FMasterDetail.fetchChilds: Detail: %s" % ( detail )
			childForm = self.getFormFactory().getInstance(detail, dbmgr=self.dbmgr)
			print "FMasterDetail.fetchChilds: childForm: %s - %s - %s" % ( childForm.getName(), childForm.getDetailTitle(), childForm.getListTitle() )
			childDbe = childForm.getDBE(self.dbmgr)
			childDbe.readFKFrom(mydbe)
			print "FMasterDetail.fetchChilds: childDbe=%s" % ( childDbe )
			tmp = self.dbmgr.search(childDbe,uselike=0)
			if isinstance(childForm, FAssociation ):
				print "FMasterDetail.fetchChilds: TODO FAssociation"
				tmp2=[]
				dest_form = None
				if detail==childForm.getFromForm():
					dest_form = childForm.getToForm()
				else:
					dest_form = childForm.getFromForm()
				dest_form = self.getFormFactory().getInstance(dest_form, dbmgr=self.dbmgr)
				dest_dbe = dest_form.getDBE(self.dbmgr)
				for assdbe in tmp:
					dest_dbe = assdbe.writeFKTo(dest_dbe)
					tmplist = self.dbmgr.search(dest_dbe,uselike=0)
					tmp2.append(tmplist[0])
				tmp=tmp2
			print "FMasterDetail.fetchChilds: tmp=%s" % ( tmp )
			self.childs[ detail ] = tmp

class FAssociation(FForm):
	"""Classe base per le form responsabili della mappatura di una DBE che rappresenta una associazione N-M su DB
	Questa associazione puo' presentare N-attributi, renderizzabili tramite fields"""
	def __init__(self,dbeassociation,from_form,to_form,nome='',azione='',metodo="POST"):
		FForm.__init__(self,nome,azione,metodo)
		self.dbeassociation=dbeassociation
		self.from_form=from_form
		self.to_form=to_form
	def getDBE(self):
		return self.dbeassociation
	def getFromForm(self):
		return self.from_form
	def getToForm(self):
		return self.to_form

class FormFactory:
	"""Returns the correct class for the given dbe name"""
	def __init__( self,verbose=False ):
		self.verbose=verbose
		self.classname2type={ "default": "FForm", }
		self.dbename2type={ "default": "FForm", }
		self.dbename2classname={ "default": "FForm", }
	def register(self,aClassName,clazz=None,dbmgr=None,package='rra.formulator'):
		istanza = None
		if not clazz is None:
			istanza = clazz('','','POST','',dbmgr)
		else:
			exec( compile("from %s import %s\nistanza = %s()"%(package,aClassName,aClassName),"formulatorcompile.py","exec") )
		self.classname2type[aClassName]=clazz #aClassName
		mydbe = istanza.getDBE(dbmgr)
		if mydbe is None:
			return
		if self.dbename2type.has_key(mydbe.getTypeName()):
			return
		self.dbename2type[mydbe.getTypeName()] = clazz #aClassName
		self.dbename2classname[mydbe.getTypeName()] = aClassName
	def getAllClassnames(self):
		return self.classname2type.keys()
	def getInstance(self,aClassname,nome='',azione='',metodo="POST",dbmgr=None):
		if self.classname2type.has_key(aClassname):
			ret = (self.classname2type[aClassname])(nome,azione,metodo,'',dbmgr)
			ret.setFormFactory(self)
			return ret
		else:
			return FForm(nome,azione,metodo,'',dbmgr)
	def getInstanceByDBEName(self,aDBEName,nome='',azione='', metodo="POST",enctype='',dbmgr=None):
		if self.dbename2type.has_key(aDBEName):
			ret = (self.dbename2type[aDBEName])(nome,azione,metodo,enctype,dbmgr)
			ret.setFormFactory(self)
			return ret
		else:
			return FForm(nome,azione,metodo)
	def getRemoteSchema(self, url, filename=None, toolkit='wxgui'):
		import xmlrpclib
		s = xmlrpclib.Server( url )
		ret = s.getFormSchema()
		src = "# -*- coding: utf-8 -*-\n"
		if toolkit>'':
			src += "from rprj.formulator.%s import *\n" % (toolkit)
		else:
			src += "from rprj.formulator import *\n"
		src += ret[0].data
		if not filename is None:
			nomemodulo = filename
			if not filename.endswith(".py"):
				nomemodulo=filename
				filename += ".py"
			else:
				nomemodulo=filename[:-3]
			file(filename,'w').write( src )
			#__import__(nomemodulo)
			return None
		#print "# ##################################"
		#print src
		#print "# ##################################"
		exec( compile(src,"remoteformschema.py","exec") )
		#print "# ##################################"
		#print dir()
		#print dir(FUser)
		#print FUser.__module__
		#print "# ##################################"
		return formschema_type_list
	def loadRemoteSchema(self, url, dbmgr=None, filename=None, toolkit='wxgui'):
		formschema_type_list = self.getRemoteSchema(url,filename,toolkit)
		if not filename is None:
			nomemodulo = filename
			if not filename.endswith(".py"):
				nomemodulo=filename
			else:
				nomemodulo=filename[:-3]
			mymodule = __import__(nomemodulo)
			formschema_type_list = mymodule.formschema_type_list
		for k in formschema_type_list.keys():
			self.register(k, formschema_type_list[k], dbmgr)

# ######################### Campi Standard

class FFileField(FField):
	def __init__(self,aNomeCampo,aTitle,aDescription,dest_directory='',aSize=1,aLength=20,aValore='',aClasseCss=None):
		"""@param dest_directory directory dove memorizzare i files"""
		FField.__init__(self,aNomeCampo,aTitle,aDescription,aSize,aLength,aValore,aClasseCss)
		self.dest_directory=dest_directory
	def generaFilename(self):
		dbe = self.myform.getDBE()
		dbe.setValuesDictionary(self.myform.getValues())
		return dbe.generaFilename()
	def render_view(self):
		raise Exception("TODO")
	def render(self):
		raise Exception("TODO")
	def readValueFromArray(self,aArray,prefix="field_"):
		raise Exception("TODO")
class FNumber(FField):
	def __init__(self,aNomeCampo,aTitle,aDescription,aValore='',aClasseCss=None,myform=None):
		FField.__init__(self,aNomeCampo,aTitle,aDescription, 0, 0,aValore,aClasseCss,myform,'n' )
class FPercent(FNumber):
	def render(self):
		raise Exception("TODO")
	def render_view(self):
		raise Exception("TODO")
class FString(FField):
	pass
class FPassword(FString):
	def render(self):
		raise Exception("TODO")
	def render_view(self):
		raise Exception("TODO")
class FServerFile(FFileField):
	def __init__(self,aNomeCampo,aTitle,aDescription,aSize,aLength=20,aValore='',aClasseCss=None,aImagePath=''):
		FFileField.__init__(self,aNomeCampo,aTitle,aDescription,aSize,aLength,aValore,aClasseCss)
		self.imagePath=aImagePath
	def render(self):
		raise Exception("TODO")
class FServerImage(FFileField):
	def __init__(self,aNomeCampo,aTitle,aDescription,aSize,aLength=20,aValore='',aClasseCss=None,aImagePath=''):
		FFileField.__init__(self,aNomeCampo,aTitle,aDescription,aSize,aLength,aValore,aClasseCss)
		self.imagePath=aImagePath
	def render(self):
		raise Exception("TODO")
class FList(FField):
	def __init__(self,aNomeCampo,aTitle,aDescription,aSize,aLength=20,aValore='',aClasseCss=None,listaValori={},altezza=1,multiselezione=False ):
		"""listaValori = {  k=>v  }
		altezza = numero di elementi visualizzati della select
		multiselezione"""
		FField.__init__(self,aNomeCampo,aTitle,aDescription,aSize,aLength,aValore,aClasseCss)
		self.listaValori=listaValori
		self.altezza=altezza
		self.multiselezione=multiselezione
	def render(self):
		raise Exception("TODO")
	def render_view(self):
		raise Exception("TODO")
class FCheckBox(FField):
	def __init__(self,aNomeCampo,aTitle,aDescription,aSize,aLength=20,aValore='',aClasseCss=None,listaValori={},multiselezione=False,stringa_separatrice="||",tipo_campo='s'):
		"""listaValori = {  k=>v  }
		altezza = numero di elementi visualizzati della select
		multiselezione
		stringa_separatrice: nel caso della multiselezione, viene effettuata una implosione dell'array di stringhe con qyesta stringa come separatore"""
		FField.__init__(self,aNomeCampo,aTitle,aDescription,aSize,aLength,aValore,aClasseCss)
		self.listaValori=listaValori
		self.multiselezione=multiselezione
		self.stringa_separatrice=stringa_separatrice
		self.tipo=tipo_campo
	def setValue(self,v):
		if self.multiselezione and type(v)==list:
			self.aValore=self.stringa_separatrice.join(v)
		else:
			self.aValore=v
	def readValueFromRequest(self,aRequest):
		if aRequest.has_key('field_%s' % self.aNomeCampo):
			self.setValue( aRequest['field_%s' % self.aNomeCampo ] )
	def render(self):
		raise Exception("TODO")
	def render_view(self):
		raise Exception("TODO")
class FTextArea(FField):
	def __init__(self,aNomeCampo,aTitle,aDescription,aSize,aValore='',aClasseCss=None,width=None,height=None,basicFormatting=True):
		"""basic_formatting:	supporto per una formattazione di base, al momento solo gli 'a capo'"""
		FField.__init__(self,aNomeCampo,aTitle,aDescription,aSize,0,aValore,aClasseCss)
		self.width=width
		self.height=height
		self.basicFormatting=basicFormatting
	def render(self):
		raise Exception("TODO")
	def render_view(self):
		raise Exception("TODO")
class FDateTime(FField):
	def __init__(self,aNomeCampo,aTitle,aDescription,aValore,aClasseCss=None,aVisualizzaData=True,aVisualizzaOra=True):
		FField.__init__(self,aNomeCampo,aTitle,aDescription,0,0,aValore,aClasseCss, myform=None, mytipo='d')
		self.aVisualizzaData = aVisualizzaData
		self.aVisualizzaOra = aVisualizzaOra
	def clean(self):
		"""Resets the field value"""
		self.setValue(None)
	def readValueFromRequest(self,aRequest):
		tmpdata = ''
		if aRequest.has_key("subfield_%s_date" % self.aNomeCampo):
			tmpdata = aRequest["subfield_%s_date" % self.aNomeCampo]
		else:
			anno=''
			if aRequest.has_key("subfield_%s_year" % self.aNomeCampo):
				anno=aRequest["subfield_%s_year" % self.aNomeCampo]
			mese=''
			if aRequest.has_key("subfield_%s_month" % self.aNomeCampo):
				mese=aRequest["subfield_%s_month" % self.aNomeCampo]
			if len(mese)==1:
				mese = "0%s" % mese
			giorno = ''
			if aRequest.has_key("subfield_%s_day" % self.aNomeCampo):
				giorno=aRequest["subfield_%s_day" % self.aNomeCampo]
			if len(giorno)==1:
				giorno = "0%s" % giorno
			if len(anno)>0 and len(mese)>0 and len(giorno)>0:
				tmpdata = "%s/%s/%s" % (anno,mese,giorno)
		tmpora=''
		ore=''
		if aRequest.has_key("subfield_%s_hour" % self.aNomeCampo):
			ore = aRequest["subfield_%s_hour" % self.aNomeCampo]
		minuti=''
		if aRequest.has_key("subfield_%s_minute" % self.aNomeCampo):
			minuti = aRequest["subfield_%s_minute" % self.aNomeCampo]
		if len(ore)>0 and len(minuti)>0:
			tmpora = "%s:%s" % (ore,minuti)
		if tmpdata!='':
			self.aValore = tmpdata
			if tmpora!='':
				self.aValore = "%s %s" % (self.aValore,tmpora)
		else:
			if self.aVisualizzaOra:
				self.aValore = tmpora
			else:
				self.aValore=''
	def render(self):
		raise Exception("TODO")
	def render_view(self):
		raise Exception("TODO")
	def render_hidden(self):
		raise Exception("TODO")
class FDateTimeReadOnly(FDateTime):
	def render(self):
		return self.render_readonly()
class FKField(FField):
	def __init__(self,aNomeCampo,aTitle,aDescription,aValore=None,aClasseCss=None,mydbe=None,myFK=None,description_columns=[],destform=None,viewmode='select',aDescription_glue=" - ",aAltezza=1,aMultiselezione=False):
		"""@param viewmode { 'select','readonly','distinct' }
		@param destform nome della classe formulator del tipo di destinazione
		@param description_columns array con le colonne descrizione della tabella puntata"""
		FField.__init__(self,aNomeCampo,aTitle,aDescription,0,0,aValore,aClasseCss)
		self.mydbe=mydbe
		self.myFK=myFK
		if not myFK is None and len(myFK)>0:
			self.myFK=myFK[0]
		self.viewmode=viewmode
		self.destform=destform
		self.description_columns=description_columns
		self.description_glue=aDescription_glue
		self.altezza=aAltezza
		self.multiselezione=aMultiselezione
		
		self.listaValori=None

	def GetListaValori(self, refresh=False):
		if self.listaValori is None or refresh:
			self.listaValori = {}
			self.listaValori[""]=""
			for myfk in self.myFK:
				cerca = self.myform.dbmgr.getClazz( myfk.tabella_riferita )
				lista = self.myform.dbmgr.search(cerca, cerca.getOrderBy())
				for dbe in lista:
					chiavi = dbe.getKeys().keys()
					chiave_array=["%s" % dbe.getValue(chiave) for chiave in chiavi]
					description_array=[dbe.getValue(chiave) for chiave in self.description_columns]
					self.listaValori[ "_".join(chiave_array) ] = self.description_glue.join(description_array)
		return self.listaValori
	def renderlink(self):
		link=""
		link_desc=""
		for myfk in self.myFK:
			cerca = self.myform.dbmgr.getClazz( myfk.tabella_riferita )
			cerca.setValue( myfk.colonna_riferita, self.aValore )
			lista = self.myform.dbmgr.search(cerca, cerca.getOrderBy())
			if len(lista)==1:
				mydbe=lista[0]
				description_array=[]
				for chiave in self.description_columns:
					description_array.append( mydbe.getValue(chiave) )
				link_desc=self.description_glue.join(description_array)
				break
		return link,link_desc

	def render_distinct(self,dbmgr):
		raise Exception("TODO")
	def render(self,dbmgr):
		if self.viewmode=='select':
			raise Exception("TODO")
		elif self.viewmode=='distinct':
			return self.render_distinct(dbmgr)
		elif self.viewmode=='readonly':
			raise Exception("TODO")
		else:
			raise Exception("TODO")
	def render_view(self,dbmgr):
		raise Exception("TODO")
	def render_readonly(self,dbmgr):
		raise Exception("TODO")
class FKObjectField(FKField):
	"""A Foreign Key pointing to multiple destinations
	@param destform se null ==> cerca tra tutte le FObject, nella form specificata altrimenti"""
	def __init__(self,aNomeCampo,aTitle,aDescription,aValore=None,aClasseCss=None,\
					mydbe=None,myFK=None,description_columns=[],destform=None,viewmode='select',aDescription_glue=" - ",aAltezza=1,aMultiselezione=False):
		FKField.__init__(self,aNomeCampo,aTitle,aDescription,aValore,aClasseCss,\
					mydbe,myFK,description_columns,destform,viewmode,aDescription_glue,aAltezza,aMultiselezione)
	def _searchObject(self,dbmgr, formFactory):
		mydbe=None
		mydestform=None
		mynomeclasse = ''
		dbeFactory = dbmgr.getFactory()
		classi = formFactory.getAllClassnames()
		if not self.myFK is None:
			for nomeclasse in classi:
				if nomeclasse=='default': continue
				exec( compile("mydestform = %s()"%(nomeclasse),"formulatorcompile.py","exec") )
				if not isinstance(mydestform,FObject): continue
				cerca = mydestform.getDBE()
				cerca.setValue( self.myFK.colonna_riferita, self.aValore )
				lista = dbmgr.search(cerca, cerca.getOrderBy())
				if len(lista)==1 and \
						lista[0].getValue(self.myFK.colonna_riferita)==cerca.getValue(self.myFK.colonna_riferita):
					mydbe=lista[0]
					mynomeclasse = nomeclasse
					break
		return [ mydbe, mydestform, nomeclasse ]
	def render_distinct(self,dbmgr):
		raise Exception("TODO")
	def render(self,dbmgr):
		raise Exception("TODO")
	def render_view(self, dbmgr, showlink=False):
		raise Exception("TODO")

class FHtml(FTextArea):
	pass
class FUuid(FString):
	pass
class FPermissions(FString):
	pass
class FLanguage(FString):
	pass
class FChildSort(FList):
	pass

# Plugin RRA
class FIPAddress(FString):
	pass




# -*- coding: utf-8 -*-
#
# @copyright &copy; 2012 by Roberto Rocco Angeloni <roberto@roccoangeloni.it>
# @license http://opensource.org/licenses/lgpl-3.0.html GNU Lesser General Public License, version 3.0 (LGPLv3)
# @version $Id: XmlrpcTransports.py $
# @package net
# 
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

#
#   Xml-Rpc Proxy
#
#   A subclass of the transport class to use behind a proxy
#
#   See: http://techweb.rfa.org/staff/bill/eXist/authpyxmlrpclib.py
#
#   Usage:
#       myProxyedTransport = HttplibTransport( <myProxyName>, <myProxyPort )
#       server = Server( <URL>, transport=myProxyedTransport )
#
# --------------------------------------------------------------------


# Import per HttplibTransport
#from xmlrpclib import *
from xmlrpclib import *
import sys
import base64

# Import per BasicAuthTransport
#import string, xmlrpclib, httplib

import string, httplib

from base64 import encodestring


__version__ = "1.0.0"


class HttplibTransport(Transport):
	'''Handles an HTTP transaction to an XML-RPC server via httplib
	(httplib includes proxy-server support)
	(C) 2003 by Roberto Rocco Angeloni'''

	def __init__(self,proxy,port):
            self.__proxy = proxy
            self.__port = port

	def request(self, host, handler, request_body, verbose=0):
            from httplib import HTTPConnection
            con = HTTPConnection( self.__proxy, self.__port )
            con.connect()
            if verbose==1 or self.verbose==1:
                print "Server:\thttp://"+host+handler
                print "Sending:\t%s" % request_body
            con.request( "POST", "http://"+host+handler, request_body )
            f = con.getresponse()
            self.verbose = verbose
            #print f.read()
            return ( self.parse_response( f ) )



class BasicAuthTransport(Transport):
    def __init__(self, username=None, password=None):
        self.username=username
        self.password=password
        self.verbose=None

    def request(self, host, handler, request_body, verbose):
        # issue XML-RPC request

        h = httplib.HTTP(host)
        h.putrequest("POST", handler)

        # required by HTTP/1.1
        h.putheader("Host", host)

        # required by XML-RPC
        h.putheader("User-Agent", self.user_agent)
        h.putheader("Content-Type", "text/xml")
        h.putheader("Content-Length", str(len(request_body)))

        # basic auth
        if self.username is not None and self.password is not None:
            h.putheader("AUTHORIZATION", "Basic %s" % string.replace(
                    base64.encodestring("%s:%s" % (self.username, self.password)),
                    "\012", ""))
        h.endheaders()

        if request_body:
            h.send(request_body)

        errcode, errmsg, headers = h.getreply()

        if errcode == 401:
            raise AuthenticationException('Authorization Required')

        if errcode != 200:
            #raise xmlrpclib.ProtocolError(
            raise ProtocolError(
                host + handler,
                errcode, errmsg,
                headers
                )

        return self.parse_response(h.getfile())

class CookieTransportSessionExpiredException(Exception):
	pass
class CookieTransport(Transport):
	def __init__(self, SESSION_ID_STRING='PHPSESSID'):
		Transport.__init__(self)
		self.mycookies=None
		self.mysessid=None
		self.SESSION_ID_STRING = SESSION_ID_STRING
	def parseCookies(self,s):
		if s is None: return {self.SESSION_ID_STRING:None}
		ret = {}
		tmp = s.split(';')
		for t in tmp:
			coppia = t.split('=')
			k = coppia[0].strip()
			v = coppia[1].strip()
			ret[k]=v
		return ret
	def single_request(self, host, handler, request_body, verbose=1):
		h = self.make_connection(host)
		if verbose:
			h.set_debuglevel(1)
		#h.accept_gzip_encoding=False
		#print "CookieTransport.request: request_body=%s"%(request_body)
		#print "CookieTransport.request: handler=%s"%(handler)
		try:
			self.send_request(h, handler, request_body)
			self.send_host(h, host)
			if not self.mysessid is None:
				h.putheader("Cookie", "%s=%s" % (self.SESSION_ID_STRING,self.mysessid) )
				#h.putheader("Cookie", "path=/")
			self.send_user_agent(h)
			self.send_content(h, request_body)
			#print "CookieTransport.request: h=%s"%(dir(h))
			resp = h.getresponse(buffering=True)
			errcode = resp.status
			errmessage = resp.reason
			headers = resp.getheaders()
			self.mycookies = self.parseCookies( resp.getheader('set-cookie') )
			if self.mysessid is None:
				if self.mycookies.has_key(self.SESSION_ID_STRING):
					self.mysessid = self.mycookies[self.SESSION_ID_STRING]
			if errcode == 200:
				self.verbose = verbose
				#try:
					#sock = h._conn.sock
				#except AttributeError:
					#sock = None
				#print "CookieTransport.request: resp=%s"%(dir(resp))
				try:
					return self.parse_response(resp)
				except Exception,e:
					print "CookieTransport.single_request: request_body=%s"%request_body
					print "CookieTransport.single_request: resp=%s" % resp.read()
					raise e
		except Fault:
			#print "CookieTransport.single_request: resp=%s" % resp.read()
			raise
		except Exception:
			# All unexpected errors leave connection in
			# a strange state, so we clear it.
			#print "CookieTransport.single_request: resp=%s" % (h.getresponse())
			self.close()
			raise
		#discard any response data and raise exception
		if (resp.getheader("content-length", 0)):
			resp.read()
		raise ProtocolError(
			host + handler,
			resp.status, resp.reason,
			resp.msg,
			)

		self.verbose=verbose

		try:
			sock = h._conn.sock
		except AttributeError:
			sock = None
		# 2011.07.08: inizio.
		return self.parse_response(resp)
		#return self._parse_response(h.getfile(), sock)
		# 2011.07.08: fine.

class CookieTransport26(Transport):
	"""Cookie Transport for Python 2.6"""
	def __init__(self, SESSION_ID_STRING='PHPSESSID'):
		Transport.__init__(self)
		self.mycookies=None
		self.mysessid=None
		self.SESSION_ID_STRING = SESSION_ID_STRING
	
	def parseCookies(self,s):
		if s is None: return {self.SESSION_ID_STRING:None}
		ret = {}
		tmp = s.split(';')
		for t in tmp:
			coppia = t.split('=')
			k = coppia[0].strip()
			v = coppia[1].strip()
			ret[k]=v
		return ret
	
	def request(self, host, handler, request_body, verbose=0):
		# issue XML-RPC request
		h = self.make_connection(host)
		if verbose:
			h.set_debuglevel(1)
		
		self.send_request(h, handler, request_body)
		self.send_host(h, host)
		if not self.mysessid is None:
			h.putheader("Cookie", "%s=%s" % (self.SESSION_ID_STRING,self.mysessid) )
			#print "CookieTransport.request: Cookie: %s=%s" % (self.SESSION_ID_STRING,self.mysessid)
		self.send_user_agent(h)
		self.send_content(h, request_body)
		
		errcode, errmsg, headers = h.getreply()
		#print "CookieTransport.request: headers=%s (%s)" % (headers, type(headers) )
		#print "CookieTransport.request: dir(headers)=%s" % ( dir(headers) )
		#print "CookieTransport.request: set-cookie=",headers.getheader('set-cookie')
		
		self.mycookies = self.parseCookies( headers.getheader('set-cookie') )
		if self.mysessid is None:
			#self.mycookies = self.parseCookies( headers.getheader('set-cookie') )
			if self.mycookies.has_key(self.SESSION_ID_STRING):
				self.mysessid = self.mycookies[self.SESSION_ID_STRING]
				#print "CookieTransport.request: self.mysessid=%s" % ( self.mysessid )
		#elif self.mycookies.has_key(self.SESSION_ID_STRING) and self.mysessid != self.mycookies[self.SESSION_ID_STRING]:
		#	self.mysessid = None
		#	raise CookieTransportSessionExpiredException("Session Expired")
		
		if errcode != 200:
			print h.getfile().read()
			raise ProtocolError(
				host + handler,
				errcode, errmsg,
				headers
				)
		
		self.verbose = verbose
		
		try:
			sock = h._conn.sock
		except AttributeError:
			sock = None
		
		return self._parse_response(h.getfile(), sock)

_major,_minor,_subversion,_stringa,_altro = sys.version_info
if _major==2 and _minor==6:
	CookieTransport = CookieTransport26

class SafeCookieTransport(CookieTransport):
	def make_connection(self, host):
		if self._connection and host == self._connection[0]:
			return self._connection[1]
		# create a HTTPS connection object from a host descriptor
		# host may be a string, or a (host, x509-dict) tuple
		try:
			HTTPS = httplib.HTTPSConnection
		except AttributeError:
			raise NotImplementedError(
				"your version of httplib doesn't support HTTPS"
				)
		else:
			chost, self._extra_headers, x509 = self.get_host_info(host)
			self._connection = host, HTTPS(chost, None, **(x509 or {}))
			return self._connection[1]



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'editwidget.ui'
#
# Created: Wed Apr 11 19:58:33 2012
#      by: PyQt4 UI code generator 4.8.5
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_EditWidget(object):
    def setupUi(self, EditWidget):
        EditWidget.setObjectName(_fromUtf8("EditWidget"))
        EditWidget.setWindowModality(QtCore.Qt.WindowModal)
        EditWidget.resize(796, 251)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(EditWidget.sizePolicy().hasHeightForWidth())
        EditWidget.setSizePolicy(sizePolicy)
        EditWidget.setWindowTitle(QtGui.QApplication.translate("EditWidget", "Form", None, QtGui.QApplication.UnicodeUTF8))
        self.verticalLayout = QtGui.QVBoxLayout(EditWidget)
        self.verticalLayout.setMargin(0)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.scrollArea = QtGui.QScrollArea(EditWidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.scrollArea.sizePolicy().hasHeightForWidth())
        self.scrollArea.setSizePolicy(sizePolicy)
        self.scrollArea.setFrameShape(QtGui.QFrame.NoFrame)
        self.scrollArea.setFrameShadow(QtGui.QFrame.Plain)
        self.scrollArea.setLineWidth(0)
        self.scrollArea.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.scrollArea.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.scrollArea.setObjectName(_fromUtf8("scrollArea"))
        self.scrollAreaWidgetContents = QtGui.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 941, 55))
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.scrollAreaWidgetContents.sizePolicy().hasHeightForWidth())
        self.scrollAreaWidgetContents.setSizePolicy(sizePolicy)
        self.scrollAreaWidgetContents.setObjectName(_fromUtf8("scrollAreaWidgetContents"))
        self.horizontalLayout_2 = QtGui.QHBoxLayout(self.scrollAreaWidgetContents)
        self.horizontalLayout_2.setMargin(0)
        self.horizontalLayout_2.setObjectName(_fromUtf8("horizontalLayout_2"))
        self.pushButton = QtGui.QPushButton(self.scrollAreaWidgetContents)
        self.pushButton.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton.setText(_fromUtf8(""))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/rprj/format-text-bold.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton.setIcon(icon)
        self.pushButton.setShortcut(QtGui.QApplication.translate("EditWidget", "Ctrl+B", None, QtGui.QApplication.UnicodeUTF8))
        self.pushButton.setObjectName(_fromUtf8("pushButton"))
        self.horizontalLayout_2.addWidget(self.pushButton)
        self.pushButton_2 = QtGui.QPushButton(self.scrollAreaWidgetContents)
        self.pushButton_2.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton_2.setText(_fromUtf8(""))
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(_fromUtf8(":/rprj/format-text-italic.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_2.setIcon(icon1)
        self.pushButton_2.setShortcut(QtGui.QApplication.translate("EditWidget", "Ctrl+I", None, QtGui.QApplication.UnicodeUTF8))
        self.pushButton_2.setObjectName(_fromUtf8("pushButton_2"))
        self.horizontalLayout_2.addWidget(self.pushButton_2)
        self.pushButton_3 = QtGui.QPushButton(self.scrollAreaWidgetContents)
        self.pushButton_3.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton_3.setText(_fromUtf8(""))
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(_fromUtf8(":/rprj/format-text-underline.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_3.setIcon(icon2)
        self.pushButton_3.setShortcut(QtGui.QApplication.translate("EditWidget", "Ctrl+U", None, QtGui.QApplication.UnicodeUTF8))
        self.pushButton_3.setObjectName(_fromUtf8("pushButton_3"))
        self.horizontalLayout_2.addWidget(self.pushButton_3)
        self.pushButton_4 = QtGui.QPushButton(self.scrollAreaWidgetContents)
        self.pushButton_4.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton_4.setText(_fromUtf8(""))
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(_fromUtf8(":/rprj/format-text-strikethrough.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_4.setIcon(icon3)
        self.pushButton_4.setObjectName(_fromUtf8("pushButton_4"))
        self.horizontalLayout_2.addWidget(self.pushButton_4)
        self.line = QtGui.QFrame(self.scrollAreaWidgetContents)
        self.line.setFrameShape(QtGui.QFrame.VLine)
        self.line.setFrameShadow(QtGui.QFrame.Sunken)
        self.line.setObjectName(_fromUtf8("line"))
        self.horizontalLayout_2.addWidget(self.line)
        self.pushButton_5 = QtGui.QPushButton(self.scrollAreaWidgetContents)
        self.pushButton_5.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton_5.setText(_fromUtf8(""))
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(_fromUtf8(":/rprj/format-text-color.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_5.setIcon(icon4)
        self.pushButton_5.setObjectName(_fromUtf8("pushButton_5"))
        self.horizontalLayout_2.addWidget(self.pushButton_5)
        self.pushButton_6 = QtGui.QPushButton(self.scrollAreaWidgetContents)
        self.pushButton_6.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton_6.setText(_fromUtf8(""))
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(_fromUtf8(":/rprj/format-fill-color.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_6.setIcon(icon5)
        self.pushButton_6.setObjectName(_fromUtf8("pushButton_6"))
        self.horizontalLayout_2.addWidget(self.pushButton_6)
        self.fontComboBox = QtGui.QFontComboBox(self.scrollAreaWidgetContents)
        self.fontComboBox.setMaximumSize(QtCore.QSize(120, 16777215))
        self.fontComboBox.setObjectName(_fromUtf8("fontComboBox"))
        self.horizontalLayout_2.addWidget(self.fontComboBox)
        self.comboFontSize = QtGui.QComboBox(self.scrollAreaWidgetContents)
        self.comboFontSize.setMaximumSize(QtCore.QSize(50, 16777215))
        self.comboFontSize.setEditable(True)
        self.comboFontSize.setObjectName(_fromUtf8("comboFontSize"))
        self.horizontalLayout_2.addWidget(self.comboFontSize)
        self.line1 = QtGui.QFrame(self.scrollAreaWidgetContents)
        self.line1.setFrameShape(QtGui.QFrame.VLine)
        self.line1.setFrameShadow(QtGui.QFrame.Sunken)
        self.line1.setObjectName(_fromUtf8("line1"))
        self.horizontalLayout_2.addWidget(self.line1)
        self.pushButton_7 = QtGui.QPushButton(self.scrollAreaWidgetContents)
        self.pushButton_7.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton_7.setText(_fromUtf8(""))
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(_fromUtf8(":/rprj/format-justify-left.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_7.setIcon(icon6)
        self.pushButton_7.setObjectName(_fromUtf8("pushButton_7"))
        self.horizontalLayout_2.addWidget(self.pushButton_7)
        self.pushButton_8 = QtGui.QPushButton(self.scrollAreaWidgetContents)
        self.pushButton_8.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton_8.setText(_fromUtf8(""))
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(_fromUtf8(":/rprj/format-justify-center.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_8.setIcon(icon7)
        self.pushButton_8.setObjectName(_fromUtf8("pushButton_8"))
        self.horizontalLayout_2.addWidget(self.pushButton_8)
        self.pushButton_9 = QtGui.QPushButton(self.scrollAreaWidgetContents)
        self.pushButton_9.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton_9.setText(_fromUtf8(""))
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(_fromUtf8(":/rprj/format-justify-right.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_9.setIcon(icon8)
        self.pushButton_9.setObjectName(_fromUtf8("pushButton_9"))
        self.horizontalLayout_2.addWidget(self.pushButton_9)
        self.pushButton_10 = QtGui.QPushButton(self.scrollAreaWidgetContents)
        self.pushButton_10.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton_10.setText(_fromUtf8(""))
        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap(_fromUtf8(":/rprj/format-justify-fill.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_10.setIcon(icon9)
        self.pushButton_10.setObjectName(_fromUtf8("pushButton_10"))
        self.horizontalLayout_2.addWidget(self.pushButton_10)
        self.line_3 = QtGui.QFrame(self.scrollAreaWidgetContents)
        self.line_3.setFrameShape(QtGui.QFrame.VLine)
        self.line_3.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_3.setObjectName(_fromUtf8("line_3"))
        self.horizontalLayout_2.addWidget(self.line_3)
        self.comboInsertList = QtGui.QComboBox(self.scrollAreaWidgetContents)
        self.comboInsertList.setObjectName(_fromUtf8("comboInsertList"))
        self.horizontalLayout_2.addWidget(self.comboInsertList)
        self.pushButton_11 = QtGui.QPushButton(self.scrollAreaWidgetContents)
        self.pushButton_11.setEnabled(False)
        self.pushButton_11.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton_11.setText(_fromUtf8(""))
        icon10 = QtGui.QIcon()
        icon10.addPixmap(QtGui.QPixmap(_fromUtf8(":/rprj/format-indent-more.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_11.setIcon(icon10)
        self.pushButton_11.setObjectName(_fromUtf8("pushButton_11"))
        self.horizontalLayout_2.addWidget(self.pushButton_11)
        self.pushButton_12 = QtGui.QPushButton(self.scrollAreaWidgetContents)
        self.pushButton_12.setEnabled(False)
        self.pushButton_12.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton_12.setText(_fromUtf8(""))
        icon11 = QtGui.QIcon()
        icon11.addPixmap(QtGui.QPixmap(_fromUtf8(":/rprj/format-indent-less.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_12.setIcon(icon11)
        self.pushButton_12.setObjectName(_fromUtf8("pushButton_12"))
        self.horizontalLayout_2.addWidget(self.pushButton_12)
        self.line_2 = QtGui.QFrame(self.scrollAreaWidgetContents)
        self.line_2.setFrameShape(QtGui.QFrame.VLine)
        self.line_2.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_2.setObjectName(_fromUtf8("line_2"))
        self.horizontalLayout_2.addWidget(self.line_2)
        self.pushButton_13 = QtGui.QPushButton(self.scrollAreaWidgetContents)
        self.pushButton_13.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton_13.setText(_fromUtf8(""))
        icon12 = QtGui.QIcon()
        icon12.addPixmap(QtGui.QPixmap(_fromUtf8(":/rprj/insert-horizontal-rule.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_13.setIcon(icon12)
        self.pushButton_13.setObjectName(_fromUtf8("pushButton_13"))
        self.horizontalLayout_2.addWidget(self.pushButton_13)
        self.pushButton_14 = QtGui.QPushButton(self.scrollAreaWidgetContents)
        self.pushButton_14.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton_14.setToolTip(QtGui.QApplication.translate("EditWidget", "Insert External Link", None, QtGui.QApplication.UnicodeUTF8))
        self.pushButton_14.setText(_fromUtf8(""))
        icon13 = QtGui.QIcon()
        icon13.addPixmap(QtGui.QPixmap(_fromUtf8(":/rprj/insert-link.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_14.setIcon(icon13)
        self.pushButton_14.setObjectName(_fromUtf8("pushButton_14"))
        self.horizontalLayout_2.addWidget(self.pushButton_14)
        self.pushButton_15 = QtGui.QPushButton(self.scrollAreaWidgetContents)
        self.pushButton_15.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton_15.setText(_fromUtf8(""))
        icon14 = QtGui.QIcon()
        icon14.addPixmap(QtGui.QPixmap(_fromUtf8(":/rprj/insert-image.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_15.setIcon(icon14)
        self.pushButton_15.setObjectName(_fromUtf8("pushButton_15"))
        self.horizontalLayout_2.addWidget(self.pushButton_15)
        spacerItem = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.verticalLayout.addWidget(self.scrollArea)
        self.textEdit = QtGui.QTextEdit(EditWidget)
        self.textEdit.setLineWrapMode(QtGui.QTextEdit.WidgetWidth)
        self.textEdit.setObjectName(_fromUtf8("textEdit"))
        self.verticalLayout.addWidget(self.textEdit)
        self.verticalLayout.setStretch(1, 1)

        self.retranslateUi(EditWidget)
        QtCore.QObject.connect(self.pushButton_2, QtCore.SIGNAL(_fromUtf8("clicked()")), EditWidget.slotItalic)
        QtCore.QObject.connect(self.pushButton_3, QtCore.SIGNAL(_fromUtf8("clicked()")), EditWidget.slotUnderline)
        QtCore.QObject.connect(self.pushButton_4, QtCore.SIGNAL(_fromUtf8("clicked()")), EditWidget.slotStrikeout)
        QtCore.QObject.connect(self.fontComboBox, QtCore.SIGNAL(_fromUtf8("currentFontChanged(QFont)")), self.textEdit.setCurrentFont)
        QtCore.QObject.connect(self.textEdit, QtCore.SIGNAL(_fromUtf8("textChanged()")), EditWidget.slotTextChanged)
        QtCore.QObject.connect(self.pushButton_5, QtCore.SIGNAL(_fromUtf8("clicked()")), EditWidget.slotTextColor)
        QtCore.QObject.connect(self.pushButton_6, QtCore.SIGNAL(_fromUtf8("clicked()")), EditWidget.slotFillColor)
        QtCore.QObject.connect(self.comboFontSize, QtCore.SIGNAL(_fromUtf8("currentIndexChanged(QString)")), EditWidget.slotFontSize)
        QtCore.QObject.connect(self.pushButton_7, QtCore.SIGNAL(_fromUtf8("clicked()")), EditWidget.slotAlignLeft)
        QtCore.QObject.connect(self.pushButton_8, QtCore.SIGNAL(_fromUtf8("clicked()")), EditWidget.slotAlignCenter)
        QtCore.QObject.connect(self.pushButton_9, QtCore.SIGNAL(_fromUtf8("clicked()")), EditWidget.slotAlignRight)
        QtCore.QObject.connect(self.pushButton_10, QtCore.SIGNAL(_fromUtf8("clicked()")), EditWidget.slotAlignFill)
        QtCore.QObject.connect(self.textEdit, QtCore.SIGNAL(_fromUtf8("cursorPositionChanged()")), EditWidget.slotCursorPositionChanged)
        QtCore.QObject.connect(self.pushButton, QtCore.SIGNAL(_fromUtf8("clicked()")), EditWidget.slotBold)
        QtCore.QObject.connect(self.comboInsertList, QtCore.SIGNAL(_fromUtf8("currentIndexChanged(int)")), EditWidget.slotInsertList)
        QtCore.QObject.connect(self.pushButton_11, QtCore.SIGNAL(_fromUtf8("clicked()")), EditWidget.slotIndent)
        QtCore.QObject.connect(self.pushButton_12, QtCore.SIGNAL(_fromUtf8("clicked()")), EditWidget.slotUnindent)
        QtCore.QObject.connect(self.pushButton_13, QtCore.SIGNAL(_fromUtf8("clicked()")), EditWidget.slotInsertHR)
        QtCore.QObject.connect(self.pushButton_14, QtCore.SIGNAL(_fromUtf8("clicked()")), EditWidget.slotInsertLink)
        QtCore.QObject.connect(self.pushButton_15, QtCore.SIGNAL(_fromUtf8("clicked()")), EditWidget.slotInsertImage)
        QtCore.QMetaObject.connectSlotsByName(EditWidget)

    def retranslateUi(self, EditWidget):
        pass

import rprj_rc



# -*- coding: utf-8 -*-

from ConfigParser import ConfigParser
import datetime, httplib, json, os, sys, traceback,  types
from base64 import b64decode,b64encode

DEBUG_JSONLIB=False

def dbeToRaw(dbe):
    ret={}
    for n in dbe.getNames():
        tmpvalue = dbe.getValue(n)
        if isinstance( tmpvalue, datetime.datetime ) or isinstance( tmpvalue, datetime.date ) or isinstance( tmpvalue, datetime.time ): # or isinstance(tmpvalue,Binary):
            ret[n] = "%s" % tmpvalue
        else:
            ret[n] = tmpvalue
    ret['_typename'] = dbe.getTypeName()
    return ret
    #return [ dbe.getTypeName(), ret ]

def dataToRaw(d):
    if type(d)==types.InstanceType:
        return dbeToRaw(d)
    elif type(d)==list or type(d)==tuple:
        return [dataToRaw(x) for x in d]
    elif type(d)==dict:
        ret = {}
        for k, v in d.items():
            ret[k] = dataToRaw(v)
        return ret
    #elif type(d)==int or type(d)==float:
    #    return "%s"%d
    return d

def rawToData(dbmgr, d):
    print d
    if type(d)==list and len(d)==2 and type(d[0]) in [str,  unicode] and type(d[1])==dict:
        print d[1]
        return rawToDbe(dbmgr, d)
    return d
    #FINIRE: conversione da liste e dizionari a dbentity
#DA QUI

def rawToDbe(dbmgr, d):
    typename = d[0]
    rawdbe = d[1]
    nomi = [ n for n in rawdbe.keys() ]
    valori = [ n for n in rawdbe.values() ]
    valori2 = []
    for v in valori:
        #if isinstance(v, Binary): v = v.data
        if dbmgr.isDateTime(v): #rra.dblayer.isDateTime(v):
            #"0000-00-00 00:00:00"
            v2=None
            try:
                v2 = datetime.datetime(int(v[:4]),int(v[5:7]),int(v[8:10]),int(v[11:13]),int(v[14:16]),int(v[17:19]))
            except Exception,e:
                try:
                    v2 = datetime.datetime(int(v[:4]),int(v[5:7]),int(v[8:10]),int(v[11:13]),int(v[14:16]))
                except Exception, e:
                    try:
                        v2 = datetime.time(int(v[11:13]),int(v[14:16]),int(v[17:19]))
                    except Exception,e:
                        v2 = datetime.time(int(v[11:13]),int(v[14:16]))
            valori2.append( v2 )
        elif (type(v)==str or type(v)==unicode) and v.startswith("base64:"):
            valori2.append( b64decode( v[len("base64:"):] ) )
        else:
            valori2.append(v)
    #if isinstance(typename, Binary): typename = typename.data
    myclazz = dbmgr.getClazzByTypeName(typename)
    #print dbmgr.getRegisteredTypes()
    #print typename, "=>", myclazz
    return myclazz( myclazz().getTableName(), names=nomi, values=valori2 )

class JSONRemoteMethod:
    def __init__(self,methodName,transport):
        self._methodName = methodName
        self._transport = transport
    def __call__(self, *args):
        #print "args:",args,len(args)
        #print "self._methodName:",self._methodName
        req = { 'method':self._methodName, 'params':args }
        return self._transport.sendRequest(req)
    def __getattr__(self,name):
        if self.__dict__.has_key(name) or name.startswith("_"):
            #print "CE L'HO",name
            return self.__dict__[name]
        self._methodName = "%s.%s" %(self._methodName,name)
        return self

class Binary:
    def __init__(self,payload):
        self._payload = payload
    def __str__(self):
        if (type(self._payload)==str or type(self._payload)==unicode) and self._payload.startswith("base64:"):
            return b64decode( self._payload[len("base64:"):] )
        return "base64:%s" % b64encode(self._payload)

class JSONClient:
    def __init__(self, host, transport=None, SESSION_ID_STRING='PHPSESSID'):
        self._protocol , tmp = host.split("://")
        tmp = tmp.split("/")
        self._host = tmp[0]
        self._port = 80
        if self._host.find(":")>=0:
            tmp2 = self._host.split(":")
            self._host = tmp2[-2]
            try:
                # TODO do this also for the xmlrpc connection :-)))
                self._port = int(tmp2[-1])
            except Exception,e:
                self._port = tmp2[-1]
        self._uri = "/%s" % "/".join(tmp[1:])
        if DEBUG_JSONLIB:
            print "JSONClient.__init__: self._protocol:",self._protocol
            print "JSONClient.__init__: self._host:",self._host
            print "JSONClient.__init__: self._port:",self._port
            print "JSONClient.__init__: self._uri:",self._uri
        self._transport=transport
        self._reqMethod="POST"
        self.mycookies=None
        self.mysessid=None
        self.SESSION_ID_STRING = SESSION_ID_STRING
        self._reqId = 0
    def __getattr__(self,name):
        if self.__dict__.has_key(name) or name.startswith("_"):
            #print "CE L'HO",name
            return self.__dict__[name]
        return JSONRemoteMethod(name,self)
    def sendRequest(self,req):
        req['id'] = self._reqId
        self._reqId += 1
        json_req = json.dumps(req)
        conn = httplib.HTTPConnection(self._host, self._port)
        req_body=json_req
        req_headers={}
        if not self.mysessid is None:
            req_headers["Cookie"] = "%s=%s" % (self.SESSION_ID_STRING,self.mysessid)
        conn.request(self._reqMethod, self._uri, req_body, req_headers)
        r1 = conn.getresponse()
        #print r1.status, r1.reason
        data1 = r1.read()
        #print dir(r1)
        headers = r1.getheaders()
        #print dir(headers)
        for h in headers:
            #print "h:",h
            nome,valore = h
            if nome=='set-cookie':
                self.mycookies = self.parseCookies(valore)
                if self.mysessid is None:
                    if self.mycookies.has_key(self.SESSION_ID_STRING):
                        self.mysessid = self.mycookies[self.SESSION_ID_STRING]
        conn.close()
        if DEBUG_JSONLIB: print "JSONClient.sendRequest: data1=%s"%data1
        try:
            return json.loads(data1)
        except Exception,e:
            print "JSONClient.sendRequest: ECCEZIONE ############# Inizio."
            print "ECCEZIONE: %s" % ( e )
            print "".join( traceback.format_tb(sys.exc_info()[2]) )
            print "JSONClient.sendRequest: ECCEZIONE ############# Data"
            print data1
            print "JSONClient.sendRequest: ECCEZIONE ############# Fine."
    def test(self):
        conn = httplib.HTTPConnection(self._host)
        #print "conn:",conn
        req_body=""
        req_headers={}
        if not self.mysessid is None:
            req_headers["Cookie"] = "%s=%s" % (self.SESSION_ID_STRING,self.mysessid)
        conn.request("GET", self._uri, req_body, req_headers)
        r1 = conn.getresponse()
        #print r1.status, r1.reason
        data1 = r1.read()
        #print dir(r1)
        headers = r1.getheaders()
        #print dir(headers)
        for h in headers:
            #print "h:",h
            nome,valore = h
            if nome=='set-cookie':
                self.mycookies = self.parseCookies(valore)
                if self.mysessid is None:
                    if self.mycookies.has_key(self.SESSION_ID_STRING):
                        self.mysessid = self.mycookies[self.SESSION_ID_STRING]
        conn.close()
        print "self.mycookies:",self.mycookies
        return data1
    def parseCookies(self,s):
        if s is None: return {self.SESSION_ID_STRING:None}
        ret = {}
        tmp = s.split(';')
        for t in tmp:
            coppia = t.split('=')
            k = coppia[0].strip()
            v = coppia[1].strip()
            ret[k]=v
        return ret


# ############################## SERVER ###################################
if __name__=='__main__':
    sys.path.insert(0,"../..")
from rprj.apps import dbschema
from rprj.dblayer import createConnection
RPRJ_CONFIG = "%s%s.config%srprj%srprj.cfg" % ( os.path.expanduser("~"), os.path.sep, os.path.sep, os.path.sep )
RPRJ_LOCAL_DB = "sqlite:%s%s.config%srprj%srprj.db" % ( os.path.expanduser("~"), os.path.sep, os.path.sep, os.path.sep )
class JSONServer:
    def __init__(self, config_file=RPRJ_CONFIG, db_url=RPRJ_LOCAL_DB):
        self.LoadConfig(config_file)
        # TODO? self.SetupPlugins(RPRJ_PLUGINS)
        # Logic
        self.dbmgr = None
        self.dbeFactory = dbschema.dbeFactory
        url = self.getPref("DB", "url", db_url)
        self.SaveConfig()
        self.connectToServer(url)
        self.SaveConfig()
    def __del__(self):
        #print "JSONServer.__del__: start."
        self.SaveConfig()
        #print "JSONServer.__del__: end."
    def ping(self, *args): #**kargv):
        return [ '', ['pong'] ]
    def echo(self,*args):
        return args[0]
    def handle_request(self, request_body):
        """req: string"""
        ret = None
        try:
            json_request = json.loads(request_body)
            if hasattr(self, json_request['method']):
                ret = getattr(self, json_request['method'])(json_request['params'])
            # TODO method with 'dots'
            elif json_request['method'].find(".")>=0:
                try:
                    pycode = "ret = self.%s(%s)" % (json_request['method'], ",".join(json_request['params']))
                    print "pycode:", pycode
                    exec(compile(pycode, "myeval.py","exec"))
                except Exception,e:
                    myerr = "%s" % e
                    print myerr
                    print "".join(traceback.format_tb(sys.exc_info()[2]))
            else:
                ret = json_request
            #ret = {'cippa':'lippa', 'bin':True, 'num':123.4, 'request': json_request, }
        except Exception,e:
            print "JSONServer.sendRequest: ECCEZIONE ############# Inizio."
            print "ECCEZIONE: %s" % ( e )
            print "".join( traceback.format_tb(sys.exc_info()[2]) )
            print "JSONServer.sendRequest: ECCEZIONE ############# Data"
            print request_body
            print "JSONServer.sendRequest: ECCEZIONE ############# Fine."
            ret = ["ERROR TODO: handle the errors!!!\nEXCEPTION: %s\n%s" %(e, "".join( traceback.format_tb(sys.exc_info()[2]) ))]
        # TODO: all the following should be in a serializer method
        print "type(ret):", type(ret), ret
        return json.dumps( dataToRaw(ret) )

# ############################## PREFERENCES ###################################
    def _checkPath(self,mycfgfile):
        dirs = mycfgfile.split(os.path.sep)[:-1]
        tmpdir = []
        for d in dirs:
            if d == "":
                continue
            tmpdir.append(d)
            tmpstr = "%s%s" % (os.path.sep, os.path.sep.join(tmpdir))
            if sys.platform=='win32':
                tmpstr = os.path.sep.join(tmpdir)
            if not os.path.exists(tmpstr):
                os.mkdir(tmpstr)
        return os.path.exists(tmpstr)
    def LoadConfig(self,config_file):
        self.config_filename = config_file
        self._checkPath(self.config_filename)
        self.config = ConfigParser()
        try:
            self.config.readfp(file(self.config_filename,'rt'))
        except Exception,e:
            print e
    def SaveConfig(self):
        self._checkPath(self.config_filename)
        self.config.write( file(self.config_filename,'w') )
    def getPref(self,section,option,default=None):
        if isinstance(default,bool): default="%s" % (default)
        if not self.config.has_section(section):
            self.config.add_section(section)
        if not self.config.has_option(section,option):
            self.config.set(section,option,default)
        tmp = self.config.get(section,option,default)
        ret = tmp
        try:
            pycode = "ret = %s" % (tmp)
            exec(compile(pycode, "myeval.py","exec"))
        except Exception,e:
            pass
        return ret
# ############################## LOGIC ###################################
    def connectToServer(self,url):
        myurl = url
#        if myurl.startswith("http"):
#            if myurl.endswith("/xmlrpc_server.php"):
#                pass # OK
#            elif myurl.endswith("/"):
#                myurl = "%sxmlrpc_server.php" % ( myurl )
#            else:
#                myurl = "%s/xmlrpc_server.php" % ( myurl )
        # Connection
        myconn = createConnection(myurl,self.getPref("Debug","connection",False))
        # Server
        self.dbmgr = dbschema.ObjectMgr(myconn, self.getPref("Debug","server",False), self.getPref('DB','schema','rprj'))
        self.dbmgr.setDBEFactory(self.dbeFactory)
    def login(self, *args):
        print "args:", args
        user = args[0][0]
        pwd = args[0][1]
        print "user:", user
        print "pwd:", pwd
        self.dbmgr.connect()
        myerr = ""
        msg = ''
        ret = None
        try:
            ret = self.dbmgr.login(user,pwd)
        except Exception,e:
            myerr = "%s" % e
            print myerr
            print "".join(traceback.format_tb(sys.exc_info()[2]))
        if not self.dbmgr.user is None:
            msg = "Logged in: %s (%s)" % (self.dbmgr.user.getValue('login'), self.dbmgr.user.getValue('fullname'))
        elif len(myerr)>0:
            msg = "Login error: %s."%myerr
        else:
            msg = "Login error!"
        return [msg, [ret]]
    def getLoggedUser(self, *args):
        ret = self.dbmgr.getDBEUser()
        if ret:
            return ['', [self.dbmgr.getDBEUser()] ]
        else:
            return [b64encode('Not logged in!'), [] ]
    def search(self, *args):
        # NOTE casesensitive NOT handled!!!
        dbe, uselike, casesensitive, orderby, ignore_deleted, full_object = args[0]
        dbe = rawToDbe(self.dbmgr, dbe)
        if orderby=='': orderby = None
        print "dbe:", dbe
        print "uselike:", uselike
        return ['', self.dbmgr.search(dbe, uselike, orderby, ignore_deleted, full_object) ]
    def insert(self, *args):
        dbe = rawToDbe(self.dbmgr, args[0][0])
        ret = self.dbmgr.insert(dbe)
        if ret is None:
            return ['', [] ]
        else:
            return ['', [ret] ]
    def update(self, *args):
        dbe = rawToDbe(self.dbmgr, args[0][0])
        ret = self.dbmgr.update(dbe)
        if ret is None:
            return ['', [] ]
        else:
            return ['', [ret] ]
    def delete(self, *args):
        dbe = rawToDbe(self.dbmgr, args[0][0])
        ret = self.dbmgr.delete(dbe)
        if ret is None:
            return ['', [] ]
        else:
            return ['', [ret] ]
    def objectById(self, *args):
        myid, ignore_deleted = args[0]
        ret = self.dbmgr.objectById(myid, ignore_deleted)
        if ret is None:
            return ['', [] ]
        else:
            return ['', [ret] ]
    def objectByName(self, *args):
        myid, ignore_deleted = args[0]
        print "myid:", myid
        ret = self.dbmgr.objectByName(myid, ignore_deleted)
        if ret is None:
            return ['', [] ]
        else:
            return ['', [ret] ]

if __name__=='__main__':
    import sys
    s = None
    if sys.platform=='darwin':
        s = JSONClient("http://localhost/~robertoroccoangeloni/rproject/plugins/jsonserver/jsonserver.php")
    else:
        s = JSONClient("http://localhost/~roberto/rproject/plugins/jsonserver/jsonserver.php")
    s = JSONClient("http://localhost:8000/")
    if os.environ.has_key('SESSION_MANAGER') and os.environ['SESSION_MANAGER'].find('roberto-ra')>=0:
        # I'm at work :-)
        print "ping:",s.ping()
        #print "s.mycookies:",s.mycookies
        print "echo:",s.echo("cippa",123.4,False,u"Lippà")
        #print "s.mycookies:",s.mycookies
        print "login:",s.login('roberto','robertopix4d')
        #print "s.mycookies:",s.mycookies
        print "dbmgr.db_version():",s.dbmgr.db_version()
        print "dbmgr.getDBEUser():",s.dbmgr.getDBEUser()
        print "dbmgr.getUserGroupsList():",s.dbmgr.getUserGroupsList()
    else:
        #print s.test()
        #print "s.mycookies:",s.mycookies
        print "ping:",s.ping()
        #print "s.mycookies:",s.mycookies
        print "echo:",s.echo("cippa",123.4,False,u"Lippà")
        #print "s.mycookies:",s.mycookies
        print "login:",s.login('roberto','echoestrade')
        #print "s.mycookies:",s.mycookies
        print "dbmgr.db_version():", rawToData( s.dbmgr, s.dbmgr.db_version() )
        print "dbmgr.getDBEUser():", rawToData( s.dbmgr, s.dbmgr.getDBEUser() )
        print "dbmgr.getUserGroupsList():", rawToData( s.dbmgr, s.dbmgr.getUserGroupsList() )




# -*- coding: utf-8 -*-

#
# @copyright &copy; 2012 by Roberto Rocco Angeloni <roberto@roccoangeloni.it>
# @license http://opensource.org/licenses/lgpl-3.0.html GNU Lesser General Public License, version 3.0 (LGPLv3)
# @version $Id: editwidgetcontroller.py $
# @package qtwidgets
# 
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

"""

pyuic4 editwidget.ui -o editwidget.py
pyrcc4 rprj.qrc -o rprj_rc.py

"""
from PyQt4 import QtCore,QtGui
import editwidget

class EditWidgetController(QtGui.QWidget):
	__EDIT_HEADER="""<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0//EN" "http://www.w3.org/TR/REC-html40/strict.dtd">
<html><head><meta name="qrichtext" content="1" /><style type="text/css">
p, li { white-space: pre-wrap; }
</style></head><body style=" font-family:'Sans Serif'; font-size:10pt; font-weight:400; font-style:normal;">
"""
	__EDIT_FOOTER="""</body></html>"""
	def __init__(self,parent,objectName):
		QtGui.QWidget.__init__(self,parent)
		self.uiEdit = editwidget.Ui_EditWidget()
		self.uiEdit.setupUi(self)
		self.setObjectName(objectName)
		self.uiEdit.comboFontSize.addItem("6")
		self.uiEdit.comboFontSize.addItem("7")
		self.uiEdit.comboFontSize.addItem("8")
		self.uiEdit.comboFontSize.addItem("9")
		self.uiEdit.comboFontSize.addItem("10")
		self.uiEdit.comboFontSize.addItem("11")
		self.uiEdit.comboFontSize.addItem("12")
		self.uiEdit.comboFontSize.addItem("14")
		self.uiEdit.comboFontSize.addItem("16")
		self.uiEdit.comboFontSize.addItem("18")
		self.uiEdit.comboFontSize.addItem("20")
		self.uiEdit.comboFontSize.addItem("22")
		self.uiEdit.comboFontSize.addItem("24")
		self.uiEdit.comboFontSize.addItem("26")
		self.uiEdit.comboFontSize.addItem("28")
		self.uiEdit.comboFontSize.addItem("36")
		self.uiEdit.comboFontSize.addItem("48")
		self.uiEdit.comboFontSize.addItem("72")
		self.uiEdit.comboFontSize.setCurrentIndex(3)
		self.uiEdit.comboInsertList.addItem( "", -1)
		self.uiEdit.comboInsertList.addItem( QtGui.QApplication.translate("EditWidgetController","Disc", None, QtGui.QApplication.UnicodeUTF8), QtGui.QTextListFormat.ListDisc)
		self.uiEdit.comboInsertList.addItem( QtGui.QApplication.translate("EditWidgetController","Circle", None, QtGui.QApplication.UnicodeUTF8), QtGui.QTextListFormat.ListCircle)
		self.uiEdit.comboInsertList.addItem( QtGui.QApplication.translate("EditWidgetController","Square", None, QtGui.QApplication.UnicodeUTF8), QtGui.QTextListFormat.ListSquare)
		self.uiEdit.comboInsertList.addItem( "123", QtGui.QTextListFormat.ListDecimal)
		self.uiEdit.comboInsertList.addItem( "abc", QtGui.QTextListFormat.ListLowerAlpha)
		self.uiEdit.comboInsertList.addItem( "ABC", QtGui.QTextListFormat.ListUpperAlpha)
		self.uiEdit.comboInsertList.setCurrentIndex(0)
	def setText(self,text):
		self.uiEdit.textEdit.setHtml(text)
	def text(self):
		ret = u"%s" % self.uiEdit.textEdit.toHtml()
		ret = ret.encode('ascii','xmlcharrefreplace')
		ret = ret.replace(EditWidgetController.__EDIT_HEADER,"").replace(EditWidgetController.__EDIT_FOOTER,"")
		ret = ret.replace("font-weight:600;","font-weight:bold;")
		start = ret.find('<body')
		if start>=0:
			ret = ret[start+1:]
			end = ret.find('>')
			ret = ret[end+1:]
		#print "EditWidgetController.text: ret=%s"%ret
		return ret
	def slotBold(self):
		if self.uiEdit.textEdit.fontWeight()==QtGui.QFont.Bold:
			self.uiEdit.textEdit.setFontWeight(QtGui.QFont.Normal)
		else:
			self.uiEdit.textEdit.setFontWeight(QtGui.QFont.Bold)
		self.uiEdit.textEdit.setFocus()
	def slotItalic(self):
		self.uiEdit.textEdit.setFontItalic( not self.uiEdit.textEdit.fontItalic() )
		self.uiEdit.textEdit.setFocus()
	def slotUnderline(self):
		self.uiEdit.textEdit.setFontUnderline( not self.uiEdit.textEdit.fontUnderline() )
		self.uiEdit.textEdit.setFocus()
	def slotStrikeout(self):
		f = self.uiEdit.textEdit.currentFont()
		f.setStrikeOut( not f.strikeOut() )
		self.uiEdit.textEdit.setCurrentFont(f)
		self.uiEdit.textEdit.setFocus()
	def slotTextChanged(self):
		#if(self.uiEdit.textEdit.toPlainText().size()==0)
			#return;
		#emit textChanged()
		#if(!this->windowTitle().endsWith(QChar('*')))
			#this->setWindowTitle( this->windowTitle().append(QChar('*')) )
		#print "EditWidgetController.slotTextChanged: TODO"
		pass
	def slotTextColor(self):
		c = QtGui.QColorDialog.getColor( self.uiEdit.textEdit.textColor() )
		self.uiEdit.textEdit.setTextColor(c)
		self.uiEdit.textEdit.setFocus()
	def slotFillColor(self):
		c = QtGui.QColorDialog.getColor( self.uiEdit.textEdit.textBackgroundColor() )
		self.uiEdit.textEdit.setTextBackgroundColor(c)
		self.uiEdit.textEdit.setFocus()
	def slotFontSize(self,s):
		mysize,success = s.toInt()
		if not success:
			return
		if mysize<=0: return
		f = self.uiEdit.textEdit.currentFont()
		f.setPointSize( mysize )
		self.uiEdit.textEdit.setCurrentFont(f)
	def slotAlignLeft(self):
		self.uiEdit.textEdit.setAlignment(QtCore.Qt.AlignLeft)
		self.uiEdit.textEdit.setFocus()
	def slotAlignCenter(self):
		self.uiEdit.textEdit.setAlignment(QtCore.Qt.AlignCenter)
		self.uiEdit.textEdit.setFocus()
	def slotAlignRight(self):
		self.uiEdit.textEdit.setAlignment(QtCore.Qt.AlignRight)
		self.uiEdit.textEdit.setFocus()
	def slotAlignFill(self):
		self.uiEdit.textEdit.setAlignment(QtCore.Qt.AlignJustify)
		self.uiEdit.textEdit.setFocus()
	def slotCursorPositionChanged(self):
		f = self.uiEdit.textEdit.currentFont();
		i = self.uiEdit.fontComboBox.findText(f.family() )
		if i>=0:
			self.uiEdit.fontComboBox.setCurrentIndex(i)
		i = self.uiEdit.comboFontSize.findText( "%s" % f.pointSize() )
		if i>=0:
			self.uiEdit.comboFontSize.setCurrentIndex(i)
	def slotInsertList(self,i):
		if i==0: return
		mystyle,success = self.uiEdit.comboInsertList.itemData(i).toInt()
		if not success:
			return
		cursor = self.uiEdit.textEdit.textCursor()
		cursor.insertList( mystyle )
		self.uiEdit.textEdit.setTextCursor(cursor)
		self.uiEdit.textEdit.setFocus()
	def slotIndent(self):
		print "EditWidgetController.slotIndent: TODO"
	def slotUnindent(self):
		print "EditWidgetController.slotUnindent: TODO"
	def slotInsertHR(self):
		cursor = self.uiEdit.textEdit.textCursor()
		cursor.insertHtml( "<hr />" )
		self.uiEdit.textEdit.setTextCursor(cursor)
		self.uiEdit.textEdit.setFocus()
	def slotInsertLink(self):
		myUrl, success = QtGui.QInputDialog.getText(self,QtGui.QApplication.translate("EditWidgetController","Insert Link", None, QtGui.QApplication.UnicodeUTF8),QtGui.QApplication.translate("EditWidgetController","Enter URL:", None, QtGui.QApplication.UnicodeUTF8),QtGui.QLineEdit.Normal,"http://")
		if not success: return
		myDesc, success = QtGui.QInputDialog.getText(self,QtGui.QApplication.translate("EditWidgetController","Insert Link", None, QtGui.QApplication.UnicodeUTF8),QtGui.QApplication.translate("EditWidgetController","Enter link description:", None, QtGui.QApplication.UnicodeUTF8),QtGui.QLineEdit.Normal,"Link")
		if not success: return
		cursor = self.uiEdit.textEdit.textCursor()
		cursor.insertHtml( "<a href=\"%s\">%s</a>" % (myUrl,myDesc) )
		self.uiEdit.textEdit.setTextCursor(cursor)
		self.uiEdit.textEdit.setFocus()
	def slotInsertImage(self):
		#imageDialog.setAcceptMode(QFileDialog::AcceptOpen);
		#//d.setFilters( myfilters );
		#imageDialog.exec();
		#QStringList l = imageDialog.selectedFiles();
		#if( l.size()<=0 )
			#return;
		#QFileInfo fi(l[0]);
		#QImage img;
		#bool loaded = img.load(l[0]);
		#if(!loaded) {
			#emit SetStatusText(tr("Unable to load image: ").append( fi.fileName() ));
			#return;
		#}
		#QTextCursor cursor = self.uiEdit.textEdit.textCursor();
		#cursor.insertImage(img,fi.fileName());
		#self.uiEdit.textEdit.setTextCursor(cursor);
		#emit ImageAdded(fi.fileName(), l[0]);
		#self.uiEdit.textEdit.setFocus();
		print "EditWidgetController.slotInsertImage: TODO"




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'mainwindow.ui'
#
# Created: Sat Jun 16 12:32:24 2012
#      by: PyQt4 UI code generator 4.9.1
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName(_fromUtf8("MainWindow"))
        MainWindow.resize(800, 600)
        MainWindow.setDockNestingEnabled(True)
        self.centralwidget = QtGui.QWidget(MainWindow)
        self.centralwidget.setObjectName(_fromUtf8("centralwidget"))
        self.verticalLayout = QtGui.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setMargin(0)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.stackedWidget = QtGui.QStackedWidget(self.centralwidget)
        self.stackedWidget.setObjectName(_fromUtf8("stackedWidget"))
        self.verticalLayout.addWidget(self.stackedWidget)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtGui.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 800, 22))
        self.menubar.setNativeMenuBar(False)
        self.menubar.setObjectName(_fromUtf8("menubar"))
        self.menuMain = QtGui.QMenu(self.menubar)
        self.menuMain.setObjectName(_fromUtf8("menuMain"))
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtGui.QStatusBar(MainWindow)
        self.statusbar.setObjectName(_fromUtf8("statusbar"))
        MainWindow.setStatusBar(self.statusbar)
        self.dockWidget = QtGui.QDockWidget(MainWindow)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.dockWidget.sizePolicy().hasHeightForWidth())
        self.dockWidget.setSizePolicy(sizePolicy)
        self.dockWidget.setFloating(False)
        self.dockWidget.setFeatures(QtGui.QDockWidget.DockWidgetMovable)
        self.dockWidget.setObjectName(_fromUtf8("dockWidget"))
        self.dockWidgetContents = QtGui.QWidget()
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.dockWidgetContents.sizePolicy().hasHeightForWidth())
        self.dockWidgetContents.setSizePolicy(sizePolicy)
        self.dockWidgetContents.setObjectName(_fromUtf8("dockWidgetContents"))
        self.verticalLayout_2 = QtGui.QVBoxLayout(self.dockWidgetContents)
        self.verticalLayout_2.setMargin(4)
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.listWidget = QtGui.QListWidget(self.dockWidgetContents)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.listWidget.sizePolicy().hasHeightForWidth())
        self.listWidget.setSizePolicy(sizePolicy)
        self.listWidget.setMinimumSize(QtCore.QSize(50, 0))
        self.listWidget.setMaximumSize(QtCore.QSize(100, 16777215))
        self.listWidget.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.listWidget.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
        self.listWidget.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
        self.listWidget.setIconSize(QtCore.QSize(50, 50))
        self.listWidget.setTextElideMode(QtCore.Qt.ElideNone)
        self.listWidget.setMovement(QtGui.QListView.Static)
        self.listWidget.setFlow(QtGui.QListView.TopToBottom)
        self.listWidget.setSpacing(25)
        self.listWidget.setViewMode(QtGui.QListView.IconMode)
        self.listWidget.setUniformItemSizes(True)
        self.listWidget.setObjectName(_fromUtf8("listWidget"))
        self.verticalLayout_2.addWidget(self.listWidget)
        self.dockWidget.setWidget(self.dockWidgetContents)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(1), self.dockWidget)
        self.actionQuit = QtGui.QAction(MainWindow)
        self.actionQuit.setObjectName(_fromUtf8("actionQuit"))
        self.actionReload_Style = QtGui.QAction(MainWindow)
        self.actionReload_Style.setObjectName(_fromUtf8("actionReload_Style"))
        self.actionLogin = QtGui.QAction(MainWindow)
        self.actionLogin.setObjectName(_fromUtf8("actionLogin"))
        self.menuMain.addAction(self.actionLogin)
        self.menuMain.addSeparator()
        self.menuMain.addAction(self.actionReload_Style)
        self.menuMain.addSeparator()
        self.menuMain.addAction(self.actionQuit)
        self.menubar.addAction(self.menuMain.menuAction())

        self.retranslateUi(MainWindow)
        QtCore.QObject.connect(self.listWidget, QtCore.SIGNAL(_fromUtf8("currentRowChanged(int)")), self.stackedWidget.setCurrentIndex)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QtGui.QApplication.translate("MainWindow", ":: R-Prj ::", None, QtGui.QApplication.UnicodeUTF8))
        self.menuMain.setTitle(QtGui.QApplication.translate("MainWindow", "Main", None, QtGui.QApplication.UnicodeUTF8))
        self.actionQuit.setText(QtGui.QApplication.translate("MainWindow", "Quit", None, QtGui.QApplication.UnicodeUTF8))
        self.actionQuit.setShortcut(QtGui.QApplication.translate("MainWindow", "Ctrl+Q", None, QtGui.QApplication.UnicodeUTF8))
        self.actionReload_Style.setText(QtGui.QApplication.translate("MainWindow", "Reload Style", None, QtGui.QApplication.UnicodeUTF8))
        self.actionReload_Style.setShortcut(QtGui.QApplication.translate("MainWindow", "Ctrl+T", None, QtGui.QApplication.UnicodeUTF8))
        self.actionLogin.setText(QtGui.QApplication.translate("MainWindow", "Login", None, QtGui.QApplication.UnicodeUTF8))
        self.actionLogin.setShortcut(QtGui.QApplication.translate("MainWindow", "Ctrl+L", None, QtGui.QApplication.UnicodeUTF8))

